<?php
/**
 * Common class.
 *
 * @since 1.0.8
 *
 * @package Envira_Audio
 * @author  Envira Team
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Common class.
 *
 * @since 1.0.8
 *
 * @package Envira_Audio
 * @author  Envira Team
 */
class Envira_Audio_Common {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.8
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.8
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.8
	 */
	public function __construct() {

		add_filter( 'envira_gallery_defaults', [ $this, 'defaults' ], 10, 2 );
		add_filter( 'envira_albums_defaults', [ $this, 'defaults' ], 10, 2 );
	}

	/**
	 * Adds the default settings for this addon.
	 *
	 * @since 1.0.8
	 *
	 * @param array $defaults  Array of default config values.
	 * @param int   $post_id     The current post ID.
	 * @return array $defaults Amended array of default config values.
	 */
	public function defaults( $defaults, $post_id ) {

		$defaults['audio']         = 0;
		$defaults['autoplay']      = 0;
		$defaults['audio_file_id'] = '';
		return $defaults;
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.8
	 *
	 * @return object The Envira_Audio_Common object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Audio_Common ) ) {
			self::$instance = new Envira_Audio_Common();
		}

		return self::$instance;
	}
}

// Load the common class.
$envira_audio_common = Envira_Audio_Common::get_instance();
