<?php
/**
 * Shortcode class.
 *
 * @since 1.0.8
 *
 * @package Envira_Audio
 * @author  Envira Team
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Shortcode class.
 *
 * @since 1.0.8
 *
 * @package Envira_Audio
 * @author  Envira Team
 */
class Envira_Audio_Shortcode {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.8
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.8
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Holds the shortcode.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $gallery_shortcode;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.8
	 */
	public function __construct() {
		if ( ! class_exists( 'Envira_Gallery_Shortcode' ) ) {
			return;
		}

		$this->base              = Envira_Audio::get_instance();
		$this->gallery_shortcode = Envira_Gallery_Shortcode::get_instance();

		add_action( 'envira_gallery_before_output', [ $this, 'styles' ], 10, 1 );
		add_action( 'envira_link_before_output', [ $this, 'styles' ] );
		add_action( 'envira_gallery_before_output', [ $this, 'scripts' ], 10, 1 );
		add_action( 'envira_link_before_output', [ $this, 'scripts' ] );

		// Gallery.
		add_action( 'envira_gallery_before_output', [ $this, 'gallery_output_css_js' ] );
		add_action( 'envira_link_before_output', [ $this, 'gallery_output_css_js' ] );
		add_filter( 'envira_gallery_toolbar_after_prev', [ $this, 'gallery_toolbar_button' ], 10, 2 );
		add_filter( 'envirabox_actions', [ $this, 'envirabox_actions' ], 90, 3 );
	}

	/**
	 * Registers and enqueues featured content styles.
	 *
	 * @since 1.0.0
	 */
	public function styles() {
		// phpcs:ignore -- Enqueue featured content styles.
		// wp_enqueue_style( $this->base->plugin_slug . '-style', plugins_url( 'assets/css/fc-admin.css', $this->base->file ), array(), $this->base->version );
	}

	/**
	 * Registers and enqueues featured content scripts.
	 *
	 * @since 1.0.0
	 *
	 * @param bool $gallery_data Gallery data.
	 */
	public function scripts( $gallery_data = false ) {

		$version = is_envira_debug_on() ? time() . '-' . ENVIRA_VERSION : ENVIRA_VERSION;

		wp_register_script( $this->base->plugin_slug . '-script', plugins_url( 'assets/js/envira-audio.js', $this->base->file ), [], $this->base->plugin_slug, $version );
		// phpcs:ignore
		// wp_register_script( $this->base->plugin_slug . '-script', plugins_url( 'assets/js/min/envira-audio-min.js', $this->base->file ), array( 'jquery' ), $this->base->version, true );

		// Enqueue featured content scripts.
		wp_enqueue_script( $this->base->plugin_slug . '-script' );

		// Localize script with audio file url.
		if ( ! empty( $gallery_data['config']['audio_file_id'] ) ) {
			$audio_file_attachment = get_post( intval( $gallery_data['config']['audio_file_id'] ) );
			if ( empty( $audio_file_attachment ) || 'audio/mpeg' !== $audio_file_attachment->post_mime_type ) {
				return;
			}
			wp_localize_script(
				$this->base->plugin_slug . '-script',
				'envira_audio_url_' . $gallery_data['id'],
				[
					'envira_audio_url' => $audio_file_attachment->guid,
				]
			);
		}
	}

	/**
	 * Enqueue CSS and JS if the Audio is enabled
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Gallery Data.
	 */
	public function gallery_output_css_js( $data ) {

		if ( ! $this->gallery_shortcode->get_config( 'audio', $data ) ) {
			return;
		}

		// Enqueue CSS + JS.
		wp_enqueue_script( $this->base->plugin_slug . '-script' );
	}

	/**
	 * Enqueue CSS and JS if the Audio is enabled
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Album Data.
	 */
	public function albums_output_css_js( $data ) {

		if ( ! $this->gallery_shortcode->get_config( 'audio', $data ) ) {
			return;
		}

		// Enqueue CSS + JS.
		wp_enqueue_script( $this->base->plugin_slug . '-script' );
	}

	/**
	 * Define JS for autoplay.
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Album Data.
	 */
	public function on_play_start( $data ) {

		$playing_value = 'false';

		if ( $this->gallery_shortcode->get_config( 'audio', $data ) && $this->gallery_shortcode->get_config( 'autoplay', $data ) ) {
			$playing_value = 'true'; } ?>

		var envira_playing = '<?php echo esc_html( $playing_value ); ?>';

		<?php
	}



	/**
	 * Outputs the audio settings for an album.
	 *
	 * @since 1.0.8
	 *
	 * @param array $data Data for the Envira Album.
	 * @return null       Return early if the audio is not enabled.
	 */
	public function album_output( $data ) {

		$instance = Envira_Albums_Shortcode::get_instance();

		if ( ! $instance->get_config( 'audio', $data ) ) {
			return;
		}

		// Output the audio init code.
		echo 'autoPlay:' . esc_attr( $instance->get_config( 'autoplay', $data ) ) . ',';
		// Note Filter.
		echo 'playSpeed:' . esc_attr( apply_filters( 'envira_audio_album_speed', $instance->get_config( 'ss_speed', $data ), $data ) ) . ',';
	}

	/**
	 * Outputs the audio button in the gallery toolbar.
	 *
	 * @since 1.0.8
	 *
	 * @param string $template  The template HTML for the gallery toolbar.
	 * @param array  $data       Data for the Envira gallery.
	 * @return string $template Amended template HTML for the gallery toolbar.
	 */
	public function gallery_toolbar_button( $template, $data ) {

		if ( ! $this->gallery_shortcode->get_config( 'audio', $data ) || 0 === intval( $this->gallery_shortcode->get_config( 'audio_file_id', $data ) ) ) {
			return $template;
		}

		// Create the audio button.
		$button = '<li><div class="envirabox-audio-button gallery_toolbar_button"><a href="#" class="envirabox-button--play" title="' . __( 'Start Audio', 'envira-audio' ) . '"></a></div></li>';

		// Return with the button appended to the template.
		return $template . $button;
	}

	/**
	 * Outputs the audio button in the gallery toolbar.
	 *
	 * @since 1.0.8
	 *
	 * @param string $template The template HTML for the gallery toolbar.
	 * @param array  $data Data for the Envira gallery.
	 * @param array  $icons Toolbar Icons.
	 *
	 * @return string $template Amended template HTML for the gallery toolbar.
	 */
	public function envirabox_actions( $template, $data, $icons = [] ) {

		// Check if audio button output is enabled, also if there is a file associated.
		if ( ! envira_get_config( 'audio', $data ) || 0 === intval( envira_get_config( 'audio_file_id', $data ) ) ) {
			return $template;
		}

		if ( isset( $icons['audio'] ) && '' !== $icons['audio'] ) {
			$button = '<div class="envirabox-audio-button"><a href="#" class="envirabox-button--play" title="' . __( 'Start Audio', 'envira-audio' ) . '">' . $icons['audio'] . '</a></div>';
		} else {
			// Build Button.
			$button = '<div class="envirabox-audio-button"><a href="#" class="envirabox-button--play" title="' . __( 'Start Audio', 'envira-audio' ) . '"></a></div>';
		}

		return $template . $button;
	}

	/**
	 * Determines if title is always shown.
	 *
	 * @since 1.0.8
	 *
	 * @param string $show  Variable for show.
	 * @param array  $data       Data for the Envira gallery.
	 * @return string $template Amended template HTML for the gallery toolbar.
	 */
	public function envira_always_show_title( $show, $data ) {

		if ( ! $this->gallery_shortcode->get_config( 'audio', $data ) || ( ! in_array( $this->gallery_shortcode->get_config( 'lightbox_theme', $data ), [ 'base_dark', 'base_light' ], true ) ) ) {
			return $show;
		}

		return true;
	}

	/**
	 * Outputs the audio button in the album toolbar.
	 *
	 * @since 1.0.4
	 *
	 * @param string $template  The template HTML for the album toolbar.
	 * @param array  $data       Data for the Envira album.
	 * @return string $template Amended template HTML for the album toolbar.
	 */
	public function album_toolbar_button( $template, $data ) {

		if ( ! Envira_Albums_Shortcode::get_instance()->get_config( 'audio', $data ) ) {
			return $template;
		}

		// Create the audio button.
		$button = '<li><a data-envirabox-play class="btnPlay" title="' . __( 'Start Audio', 'envira-audio' ) . '" href="javascript:;"></a></li>';

		// Return with the button appended to the template.
		return $template . $button;
	}

	/**
	 * Provide method to filter speed for galleries
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Gallery Data.
	 * @param array $gallery_id Gallery ID.
	 */
	public function change_gallery_ss_speed( $data, $gallery_id ) {

		if ( ! $this->gallery_shortcode->get_config( 'audio', $data ) ) {
			return $data;
		}

		$data['config']['ss_speed'] = ! empty( $data['config']['ss_speed'] ) ? apply_filters( 'envira_audio_gallery_speed', $data['config']['ss_speed'], $data ) : apply_filters( 'envira_audio_gallery_speed', false, $data );

		return $data;
	}

	/**
	 * Provide method to filter speed for albums
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Gallery Data.
	 * @param array $album_id Album ID.
	 */
	public function change_album_ss_speed( $data, $album_id ) {

		if ( ! function_exists( 'envira_albums_get_config' ) || ! envira_albums_get_config( 'audio', $data ) ) {
			return $data;
		}

		$ss_speed = ( isset( $data['config']['ss_speed'] ) ) ? $data['config']['ss_speed'] : false;

		$data['config']['ss_speed'] = apply_filters( 'envira_audio_album_speed', $ss_speed, $data );

		return $data;
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.8
	 *
	 * @return object The Envira_Audio_Shortcode object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Audio_Shortcode ) ) {
			self::$instance = new Envira_Audio_Shortcode();
		}

		return self::$instance;
	}
}

// Load the shortcode class.
$envira_audio_shortcode = Envira_Audio_Shortcode::get_instance();
