<?php
/**
 * Beaver Builder Module Class
 *
 * @package Envira Proofing
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Envira Basic Module Class.
 */
class Envira_Basic_Module extends FLBuilderModule {

	/**
	 * Constructor function for the module. You must pass the
	 * name, description, dir and url in an array to the parent class.
	 *
	 * @method __construct
	 */
	public function __construct() {
		parent::__construct(
			[
				'name'            => __( 'Envira Gallery', 'envira-beaver-builder' ),
				'description'     => __( 'An envira for coding new modules.', 'envira-beaver-builder' ),
				'category'        => __( 'Envira Gallery Modules', 'envira-beaver-builder' ),
				'dir'             => ENVIRA_BB_DIR . 'modules/envira/',
				'url'             => ENVIRA_BB_URL . 'modules/envira/',
				'editor_export'   => true, // Defaults to true and can be omitted.
				'enabled'         => true, // Defaults to true and can be omitted.
				'partial_refresh' => true, // Set this to true to enable partial refresh.
			]
		);

		/**
		 * Use these methods to enqueue css and js already
		 * registered or to register and enqueue your own.
		 */

		$this->add_js( 'envira-lib', $this->url . 'js/frontend.js', [], '', true );
	}

	/**
	 * Returns Envira SVG Icon
	 *
	 * @since 1.9.0
	 *
	 * @param string $icon Icon.
	 */
	public function get_icon( $icon = '' ) {
		return '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path style="fill:#7cc048;" d="M0 32c477.6 0 366.6 317.3 367.1 366.3L448 480h-26l-70.4-71.2c-39 4.2-124.4 34.5-214.4-37C47 300.3 52 214.7 0 32zm79.7 46c-49.7-23.5-5.2 9.2-5.2 9.2 45.2 31.2 66 73.7 90.2 119.9 31.5 60.2 79 139.7 144.2 167.7 65 28 34.2 12.5 6-8.5-28.2-21.2-68.2-87-91-130.2-31.7-60-61-118.6-144.2-158.1z"/></svg>';
	}

	/**
	 * Use this method to work with settings data before
	 * it is saved. You must return the settings object.
	 *
	 * @method update
	 * @param object $settings The settings.
	 */
	public function update( $settings ) {
		return $settings;
	}

	/**
	 * This method will be called by the builder
	 * right before the module is deleted.
	 *
	 * @method delete
	 */
	public function delete() {

	}

	/**
	 * Add additional methods to this class for use in the
	 * other module files such as preview.php, frontend.php
	 * and frontend.css.php.
	 *
	 * @method envira_method
	 */
	public static function envira_galleries() {

		$gallery_dropdown = false;

		if ( function_exists( 'envira_get_galleries' ) ) {
			$galleries = envira_get_galleries( false, true );
			if ( ! empty( $galleries ) ) :
				$gallery_dropdown   = [];
				$gallery_dropdown[] = '';
				foreach ( $galleries as $gallery ) {
					$gallery_dropdown[ $gallery['id'] ] = $gallery['config']['title'];
				}
				// Sort Galleries.
				natsort( $gallery_dropdown );
			endif;
		}

		return $gallery_dropdown;

	}

	/**
	 * Populate margin dropdown.
	 */
	public static function gallery_margins() {

		$margin_dropdown           = [];
		$margin_dropdown[0]        = 'None';
		$margin_dropdown[1]        = 'Small';
		$margin_dropdown[5]        = 'Medium';
		$margin_dropdown[20]       = 'Large';
		$margin_dropdown['custom'] = 'Custom';

		return $margin_dropdown;

	}

	/**
	 * Populate column dropdown.
	 */
	public static function envira_get_columns() {

		$column_dropdown = false;

		if ( function_exists( 'envira_get_layouts' ) ) {
			$columns            = envira_get_columns();
			$column_dropdown    = [];
			$column_dropdown[0] = '&nbsp;&nbsp;';
			foreach ( $columns as $column ) {
				$column_dropdown[ $column['value'] ] = $column['name'];
			}
			// Sort Galleries.
			natsort( $column_dropdown );
		}

		return $column_dropdown;

	}

	/**
	 * Populate layout dropdown.
	 */
	public static function envira_get_layouts() {

		$layout_dropdown = false;

		if ( function_exists( 'envira_get_layouts' ) ) {
			$layouts            = envira_get_layouts();
			$layout_dropdown    = [];
			$layout_dropdown[0] = '&nbsp;&nbsp;';
			foreach ( $layouts as $slug => $layout ) {
				$layout_dropdown[ $slug ] = $layout['name'];
			}
			// Sort Galleries.
			natsort( $layout_dropdown );
		}

		return $layout_dropdown;

	}

	/**
	 * Populate lightbox themes dropdown.
	 */
	public static function envira_lightbox_themes() {

		$theme_dropdown = false;

		if ( function_exists( 'envira_get_galleries' ) ) {
			$themes            = envira_get_lightbox_themes();
			$theme_dropdown    = [];
			$theme_dropdown[0] = 'No lightbox';
			foreach ( $themes as $theme ) {
				$theme_dropdown[ $theme['value'] ] = $theme['name'];
			}
			// Sort Galleries.
			natsort( $theme_dropdown );
		}

		return $theme_dropdown;

	}

}

/**
 * Register the module and its form settings.
 */
FLBuilder::register_module(
	'Envira_Basic_Module',
	[
		'general' => [ // Tab.
			'title'    => __( 'General', 'envira-beaver-builder' ), // Tab title.
			'sections' => [ // Tab Sections.
				'gallery'  => [ // Section.
					'title'  => __( 'Gallery', 'envira-beaver-builder' ), // Section Title.
					'fields' => [
						'envira_gallery_id' => [
							'type'    => 'select',
							'label'   => __( 'Select Gallery', 'envira-beaver-builder' ),
							'default' => '',
							'options' => Envira_Basic_Module::envira_galleries(),
						],
						'gallery_layout'    => [
							'type'    => 'select',
							'label'   => __( 'Select Layout', 'envira-beaver-builder' ),
							'default' => '',
							'options' => Envira_Basic_Module::envira_get_layouts(),
						],
						'gallery_columns'   => [
							'type'    => 'select',
							'label'   => __( 'Select Columns', 'envira-beaver-builder' ),
							'default' => '',
							'options' => Envira_Basic_Module::envira_get_columns(),
						],
						'gallery_margins'   => [
							'type'    => 'select',
							'label'   => __( 'Gallery Margins', 'envira-beaver-builder' ),
							'default' => '',
							'options' => Envira_Basic_Module::gallery_margins(),
						],
						'custom_margin'     => [
							'type'        => 'unit',
							'label'       => __( 'Custom Margin', 'envira-beaver-builder' ),
							'description' => 'px',
							'default'     => '',
						],
					],
				],
				'lightbox' => [ // Section.
					'title'  => __( 'Lightbox', 'envira-beaver-builder' ), // Section Title.
					'fields' => [
						'lightbox_theme' => [
							'type'    => 'select',
							'label'   => __( 'Select Lightbox Theme', 'envira-beaver-builder' ),
							'default' => 'No lightbox',
							'options' => Envira_Basic_Module::envira_lightbox_themes(),
						],
					],
				],
			],
		],
	]
);
