<?php
/**
 * Beaver Builder Frontend PHP
 *
 * @package Envira Beaver Builder
 */

// setup debug and global variables.
$envira_debug = ( defined( 'ENVIRA_DEBUG' ) && 'true' === ENVIRA_DEBUG ) ? $version = time() . '-' . ENVIRA_VERSION : ENVIRA_VERSION;

// get gallery ID from settings if it exists, then gallery data.
$gallery_id   = ! empty( $settings->envira_gallery_id ) ? intval( $settings->envira_gallery_id ) : false;
$gallery_data = ( false !== $gallery_id ) ? envira_get_gallery( $gallery_id, true ) : false; // flush cache while we do this with 'true'.

// determine layout from the info you have.
$layout = isset( $settings->gallery_layout ) ? $settings->gallery_layout : false;
$layout = ( false === $layout && ! empty( $gallery_data['config']['layout'] ) ) ? $gallery_data['config']['layout'] : $layout;
$layout = ( false === $layout ) ? 'automattic' : $layout;

// Debug.
if ( $envira_debug ) {
	error_log( 'layout : ' . $layout );
	error_log( 'settings->gallery_layout : ' . $settings->gallery_layout );
	if ( ! empty( $gallery_data['config']['layout'] ) ) {
		error_log( 'gallery_data layout : ' . $gallery_data['config']['layout'] );
	}
}

// Setup remaining gallery vars and shortcode.
if ( 'automatic' === $layout ) {
	$columns = '0';
} else {
	$columns = isset( $settings->gallery_columns ) && ! empty( $settings->gallery_columns ) ? $settings->gallery_columns : 2;
}
$lightbox        = ! isset( $settings->lightbox_theme ) || '0' === $settings->lightbox_theme ? false : 1;
$lightbox_theme  = isset( $settings->lightbox_theme ) ? $settings->lightbox_theme : 'base_dark';
$isotope         = isset( $settings->enable_isotope[0] ) && 'on' === $settings->enable_isotope[0] ? 1 : false;
$gallery_margins = isset( $settings->gallery_margins ) ? $settings->gallery_margins : false;
$custom_margin   = isset( $settings->custom_margin ) ? $settings->custom_margin : '';

if ( strtolower( $gallery_margins ) === 'custom' ) {
	$margin = intval( $custom_margin );
} else {
	$margin = intval( $gallery_margins );
}

error_log( 'margin : ' . $margin );
error_log( 'gallery_margins : ' . $gallery_margins );
error_log( 'custom_margin : ' . $custom_margin );


$shortcode   = ( '[envira-gallery id="' . intval( $gallery_id ) . '" type="beaverbuilder" cache="false" layout="' . $layout . '" columns="' . $columns . '" lightbox_enabled="' . $lightbox . '" lightbox_theme="' . $lightbox_theme . '" isotope="' . $isotope . '" margin="' . $margin . '" ]' );
$shortcode_s = ( 'id="' . intval( $gallery_id ) . '" type="beaverbuilder" cache="false" layout="' . $layout . '" columns="' . $columns . '" lightbox_enabled="' . $lightbox . '" lightbox_theme="' . $lightbox_theme . '" isotope="' . $isotope . '" margin="' . $margin . '" ' );

// Debug.
if ( $envira_debug ) {
	error_log( 'shortcode : ' . $layout );
	error_log( print_r( $shortcode_s, true ) );
}

// output gallery HTML via shortcode.
echo do_shortcode( $shortcode );

// Debug.
if ( $envira_debug ) {
	error_log( 'checking to see if we need to update ' );
	error_log( $gallery_id );
}

// Create hidden HTML input fields.
if ( $gallery_id && $gallery_id > 0 ) {
	$gallery                  = envira_get_gallery( $gallery_id );
	$columns_to_output        = $gallery['config']['columns'];
	$layout_to_output         = $gallery['config']['layout'];
	$margin_to_output         = $gallery['config']['margin'];
	$lightbox_to_output       = $gallery['config']['lightbox_enabled'];
	$lightbox_theme_to_output = $gallery['config']['lightbox_theme'];
} else {
	$columns_to_output        = '';
	$layout_to_output         = '';
	$margin_to_output         = '';
	$lightbox_to_output       = '';
	$lightbox_theme_to_output = '';
}

echo '<input type="hidden" id="envira-bb-gallery-id" name="envira-bb-gallery-id" value="' . intval( $gallery_id ) . '" />';
echo '<input type="hidden" id="envira-bb-columns" name="envira-bb-columns" value="' . esc_html( $columns_to_output ) . '" />';
echo '<input type="hidden" id="envira-bb-layout" name="envira-bb-layout" value="' . esc_html( $layout_to_output ) . '" />';
echo '<input type="hidden" id="envira-bb-margin" name="envira-bb-margin" value="' . esc_html( $margin_to_output ) . '" />';
echo '<input type="hidden" id="envira-bb-lightbox-enabled" name="envira-bb-lightbox-enabled" value="' . esc_html( $lightbox_to_output ) . '" />';
echo '<input type="hidden" id="envira-bb-lightbox-theme" name="envira-bb-lightbox-theme" value="' . esc_html( $lightbox_theme_to_output ) . '" />';

// Debug.
if ( $envira_debug ) {
	error_log( 'before gallery data: ' );
	$debug_gallery_data = get_post_meta( $gallery_id, '_eg_gallery_data' );
	error_log( print_r( $debug_gallery_data, true ) );
}

// Update gallery info in database with any adjusted settings.
if ( $gallery_id && $gallery_id > 0 ) {
	if ( ! empty( $gallery_data['config'] ) ) {
		// Debug.
		if ( $envira_debug ) {
			error_log( 'updating STUFF ****** ' );
			error_log( 'updating to ' . $layout . ' with ' . $columns . ' columns ' );
		}
		$gallery_data['config']['layout']           = $layout;
		$gallery_data['config']['columns']          = $columns;
		$gallery_data['config']['margin']           = $margin;
		$gallery_data['config']['lightbox_enabled'] = ( '0' === $lightbox_theme ) ? false : true; // $lightbox;
		$gallery_data['config']['lightbox_theme']   = ( '0' === $lightbox_theme ) ? 'base_dark' : $lightbox_theme;
		// Debug.
		if ( $envira_debug ) {
			error_log( 'updating STUFF lightbox: ' . $lightbox );
			error_log( print_r( $settings, true ) );
		}
	}
	// Debug.
	if ( $envira_debug ) {
		error_log( 'updating gallery ****** ' );
		error_log( print_r( $settings, true ) );
		error_log( print_r( $gallery_data, true ) );
	}
	// Update gallery data.
	update_post_meta( $gallery_id, '_eg_gallery_data', $gallery_data );
}
