<?php
/**
 * Envira Elementor Global Functions.
 *
 * @since 1.0.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

add_filter( 'envira_gallery_custom_gallery_data_by_elementor', 'envira_elementor_update_gallery_data', 10, 4 );

/**
 * Register the widget controls.
 *
 * @since 1.0.0
 *
 * @access public
 * @param array   $gallery_data Envira gallery data.
 * @param array   $atts Atts passed.
 * @param array   $post Post info.
 * @param integer $gallery_id ID of gallery.
 * @return array gallery data.
 */
function envira_elementor_update_gallery_data( $gallery_data, $atts, $post, $gallery_id ) {

	if ( ! isset( $atts['type'] ) || 'elementor' !== $atts['type'] ) {
		return $gallery_data;
	}
	if ( isset( $atts['columns'] ) && ! empty( $atts['columns'] ) ) {
		$gallery_data['config']['columns'] = intval( $atts['columns'] );
	}
	if ( isset( $atts['layout'] ) && ! empty( $atts['layout'] ) ) {
		$gallery_data['config']['layout'] = esc_html( $atts['layout'] );
	}
	if ( isset( $atts['lightbox_enabled'] ) && ! empty( $atts['lightbox_enabled'] ) ) {
		$gallery_data['config']['lightbox_enabled'] = esc_html( $atts['lightbox_enabled'] );
	}
	if ( isset( $atts['lightbox_theme'] ) && ! empty( $atts['lightbox_theme'] ) ) {
		$gallery_data['config']['lightbox_theme'] = esc_html( $atts['lightbox_theme'] );
	}
	if ( isset( $atts['isotope'] ) && ! empty( $atts['isotope'] ) ) {
		$gallery_data['config']['isotope'] = esc_html( $atts['isotope'] );
	}

	return $gallery_data;
}


add_filter( 'envira_gallery_shortcode_start', 'envira_elementor_shortcode_nobail', 999, 2 );
/**
 * Register the widget controls.
 *
 * Adds different input fields to allow the user to change and customize the widget settings.
 *
 * @since 1.0.0
 *
 * @access public
 * @param array $response Response.
 * @param array $args Args passed.
 * @return array gallery data.
 */
function envira_elementor_shortcode_nobail( $response, $args ) {
	if ( isset( $args['type'] ) && 'elementor' === $args['type'] ) {
		$response           = [];
		$response['action'] = 'nobail';
	}
	return $response;
}
