<?php
/**
 * Cron Jobs
 *
 * @since 1.5.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Instagram\Frontend;

use Envira\Instagram\Frontend\Shortcode;
use Envira\Instagram\Utils\Data_Encryption;
use WP_Error;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {

	exit;

}

/**
 * Cron Job Class.
 *
 * @since 1.5.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */
class CronJobs {

	/**
	 * The time before it takes to refresh long term tickets (which expire at 90 days)
	 *
	 * @since 1.5.0
	 *
	 * @var string
	 */
	public $days_before_renewing = 1;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.5.0
	 */
	public function __construct() {
	}

	/**
	 * XXXXX
	 *
	 * @since 1.5.0
	 *
	 * @return string Revised Where.
	 */
	public function check_long_term_token() {

		do_action( 'envira_instagram_before_token_check' );

		$auth           = [];
		$total_accounts = 3;
		$slots          = [];
		$encryption     = new \Envira\Instagram\Utils\Data_Encryption();

		// go through the IG tokens and see if anything is going to expire within X days.
		for ( $x = 1; $x <= $total_accounts; $x++ ) {
			$temp_auth = envira_instagram_get_instagram_auth( $x );
			if ( $temp_auth ) {
				$slots[] = $temp_auth;
			}
		}

		if ( empty( $slots ) ) {
			return false;
		}

		$open_slot = 1;

		// check each token, renew if we need to.
		foreach ( $slots as $slot ) {

			$token = $encryption->maybe_decrypt( $slot['token'] );

			if ( empty( $token ) ) {
				continue;
			}

			$time_taken = ! empty( $slot['time_taken'] ) ? $slot['time_taken'] : 0;
			$difference = ! empty( $slot['time_taken'] ) ? time() - $slot['time_taken'] : time();
			$days       = floor( $difference / ( 60 * 60 * 24 ) );

			// allow a filter to override the addon's days before renewing limit.
			$days_before_renewing = apply_filters( 'envira_instagram_long_token_days_before_renewing', $this->days_before_renewing, $slot );

			if ( $time_taken > 0 && $days < $days_before_renewing ) {
				// no need to renew.
				continue;
			}
			// otherwise, renew.

			$response = wp_safe_remote_get(
				'https://graph.instagram.com/refresh_access_token?grant_type=ig_refresh_token&access_token=' . $token
			);

			$response_code = wp_remote_retrieve_response_code( $response );
			$response_body = wp_remote_retrieve_body( $response );
			$response_data = json_decode( $response_body );

			if ( 200 !== intval( $response_code ) ) {
				// Single cron event to delete the data.
				if ( ! wp_next_scheduled( 'remove_instagram_data_by_slot' ) ) {
					wp_schedule_single_event( time() + WEEK_IN_SECONDS, 'remove_instagram_data_by_slot', [ $slot['username'] ] );
				}

				continue;
			}

			if ( ! empty( $response_data->access_token ) && ! empty( $response_data->expires_in ) ) {
				$slot['time_taken'] = time();
				$slot['expires_in'] = $response_data->expires_in;
				$slot['token']      = $encryption->maybe_encrypt( $response_data->access_token );
			}

			if ( 1 === $open_slot ) {
				update_option( 'envira_instagram', $slot );
			} else {
				update_option( 'envira_instagram_' . $open_slot, $slot );
			}
		}

		do_action( 'envira_instagram_after_token_check' );
	}
}
