const EnviraProofing = {};

EnviraProofing.theImageIds = [];
EnviraProofing.theImages = [];
EnviraProofing.cookies = Cookies.noConflict();
EnviraProofing.galleryId = 0;

// Proofing fields on each image.
EnviraProofing.toggleFields = checkboxElement => {
	const isChecked = checkboxElement.checked;
	const proofingFields = checkboxElement.parentElement.querySelector('.envira-proofing-fields');
	const image = checkboxElement.closest('.envira-gallery-item-inner').querySelector('img');
	const imageId = image.dataset.enviraItemId;
	const galleryId = image.dataset.enviraGalleryId;
	const galleryIdLabel = parseInt(galleryId ?? 0);
	const cookieImageIdsFieldName = 'envira_proof_image_ids_' + galleryIdLabel;
	const cookieImageImagesFieldName = 'envira_proof_images_' + galleryIdLabel;

	if(isChecked){
		if(!!proofingFields){
			proofingFields.style.display = 'block';
		}
		checkboxElement.closest('.envira-gallery-item-inner').classList.add('envira-proofing-selected');

		const imageSrc = image.getAttribute('src');
		const count = document.querySelectorAll(`div.envira-proofing-summary-box-inner div.images-inner div.image img[src="${imageSrc}"]`).length;

		if(0 === count){
			document.querySelector('div.envira-proofing-summary-box-inner div.images-inner')
				.insertAdjacentHTML('beforeend', `<div class="image" data-src="${imageSrc}"><img src="${imageSrc}" /></div>`);
		}

		const theImageId = JSON.stringify(imageId);
		const theImage = JSON.stringify(imageSrc);

		if(!EnviraProofing.theImageIds.includes(theImageId)){
			EnviraProofing.theImageIds.push(theImageId);
		}

		EnviraProofing.cookies.set(cookieImageIdsFieldName, JSON.stringify(EnviraProofing.theImageIds), {expires: 9999999, path: '/'});

		if(!EnviraProofing.theImages.includes(theImage)){
			EnviraProofing.theImages.push(theImage);
		}

		EnviraProofing.cookies.set(cookieImageImagesFieldName, JSON.stringify(EnviraProofing.theImages), {expires: 9999999, path: '/'});
	} else {
		const theImageId = JSON.stringify(imageId);
		const theImage = JSON.stringify(image.getAttribute('src'));

		const index = EnviraProofing.theImageIds.indexOf(theImageId);
		if(index !== - 1){
			EnviraProofing.theImageIds.splice(index, 1);
		}

		EnviraProofing.cookies.set(cookieImageIdsFieldName, JSON.stringify(EnviraProofing.theImageIds), {expires: 9999999, path: '/'});

		const imageIndex = EnviraProofing.theImages.indexOf(theImage);
		if(imageIndex !== - 1){
			EnviraProofing.theImages.splice(imageIndex, 1);
		}

		EnviraProofing.cookies.set(cookieImageImagesFieldName, JSON.stringify(EnviraProofing.theImages), {expires: 9999999, path: '/'});

		if(!!proofingFields){
			proofingFields.style.display = 'none';
		}

		checkboxElement.closest('.envira-gallery-item-inner').classList.remove('envira-proofing-selected');

		// Remove from summary
		const imageSrc = image.getAttribute('src');
		const imageToRemove = document.querySelector(`div.envira-proofing-summary-box-inner div.images-inner div.image img[src="${imageSrc}"]`);
		if(imageToRemove){
			imageToRemove.parentElement.remove();
		}
	}

	// If isotope is enabled, re-layout so the fields display
	const enviraContainer = checkboxElement.closest('.envira-gallery-public');
	if(enviraContainer.classList.contains('enviratope')){
		jQuery(enviraContainer).enviraImagesLoaded().done(() => {
			jQuery(enviraContainer).enviratope('layout');
		});
	}

	// Toggle summary bar.
	const imagesCount = document.querySelectorAll('div.envira-proofing-summary-box-inner div.images div.image').length;
	if(imagesCount === 0){
		document.querySelector('div.envira-proofing-summary-box-inner').style.bottom = '-120px';
	} else {
		document.querySelector('div.envira-proofing-summary-box-inner').style.bottom = '0';
	}
}

EnviraProofing.toggleFieldsObserver = () => {
	const galleryItems = document.querySelectorAll('form.envira-proofing-form .envira-gallery-item');
	galleryItems.forEach((galleryItem) => {
		const checkboxElement = galleryItem.querySelector('input.envira-proofing-select-image');
		if(!checkboxElement){
			return;
		}
		// Toggle fields on page load.
		EnviraProofing.toggleFields(checkboxElement);
		checkboxElement.addEventListener('change', () => {
			EnviraProofing.toggleFields(checkboxElement);
		});
	});
}

// Delete button clear cookies.
EnviraProofing.buttonsObservers = () => {
	const deleteButton = document.querySelector('input.envira_proofing_delete');
	const submitButton = document.querySelector('.envira-proofing-summary-box button[name=envira_proofing_submit]');
	const saveButton = document.querySelector('.envira-proofing-summary-box button[name=envira_proofing_save]');
	if(deleteButton){
		deleteButton.addEventListener('click', event => {
			// Delete all cookies, if the user is not allowed to delete, cookies will be refreshed with the related order.
			const galleryId = event.target.closest('form').dataset.enviraId;
			[
				'envira_proof_image_ids_0',
				'envira_proof_images_0',
				'envira_proof_image_ids_' + galleryId,
				'envira_proof_images_' + galleryId,
			].forEach((name) => {
				EnviraProofing.cookies.remove(name, {path: '/'});
			});
		});
	}
	if(submitButton && saveButton){
		submitButton.addEventListener('click', event => {
			document.querySelector('form.envira-proofing-form input[name=envira_proofing_submit]').click()
		});
		saveButton.addEventListener('click', event => {
			document.querySelector('form.envira-proofing-form input[name=envira_proofing_save]').click()
		});
	}
};

// Proofing checkbox on lightbox sync with gallery.
EnviraProofing.changeLBCheckbox = (element) => {
	const lightboxFields = element.closest('form.envira-proofing-form').querySelector('.envira-proofing-fields');
	const itemId = element.dataset.itemId;
	const galleryElement = document.querySelector(`input[type="checkbox"][name="envira_proofing[images][${itemId}]"]`);

	if(element.checked){
		lightboxFields.style.display = 'block';
	} else {
		lightboxFields.style.display = 'none';
	}

	galleryElement.checked = element.checked;
	EnviraProofing.toggleFields(galleryElement);
}

// Proofing size number input on lightbox sync with gallery.
EnviraProofing.changeLBFieldsSizesQuantityNumber = (element) => {
	const itemId = element.dataset.itemId;
	const size = element.dataset.size;
	const galleryElement = document.querySelector(`input[type="number"][name="envira_proofing[quantities][${itemId}][${size}]"]`);
	galleryElement.value = element.value;
}

// Proofing quantities number input on lightbox sync with gallery.
EnviraProofing.changeLBFieldsQuantitiesNumber = (element) => {
	const itemId = element.dataset.itemId;
	const galleryElement = document.querySelector(`input[type="number"][name="envira_proofing[quantities][${itemId}]"]`);
	galleryElement.value = element.value;
}

// Proofing size checkbox on lightbox sync with gallery.
EnviraProofing.changeLBFieldsSizesCheckbox = (element) => {
	const itemId = element.dataset.itemId;
	const size = element.dataset.size;
	const galleryElement = document.querySelector(`input[type="checkbox"][name="envira_proofing[quantities][${itemId}][${size}]"]`);
	galleryElement.checked = element.checked;
}

EnviraProofing.lightboxObserver = () => {
	// This is a jQuery registered event, it needs jQuery to be detected.
	jQuery(document).on('envirabox_api_after_show', (e, obj, instance, current) => {
		const enviraProofingOrderId = parseInt(EnviraProofing.cookies.get('envira_proofing_order_id') ?? 0);

		const configProofing = !!obj.get_config('proofing');
		const configProofingLightbox = !!obj.get_config('proofing_lightbox');
		const configProofingMultipleEnabled = !!obj.get_config('proofing_multiple_enabled');
		if((!configProofing || !configProofingLightbox) || (configProofingMultipleEnabled && (enviraProofingOrderId > 0))){
			return;
		}

		document.querySelector('.envirabox-container').classList.add('envirabox-proofing');

		const quantityEnabled = !!obj.get_config('proofing_quantity_enabled');
		const sizesEnabled = !!obj.get_config('proofing_size_enabled');
		const sizes = obj.get_config('proofing_sizes');
		const validSizes = (Object.keys(sizes).length > 0) && sizesEnabled;
		let output = '';

		const itemId = current.enviraItemId;
		const proofing_add_to_order_label = obj.get_config('proofing_add_to_order_label') ?? 'Add To Order';

		// Obtain the equivalent field from the gallery.
		const galleryCheckbox = document.querySelector(`input[type="checkbox"][name="envira_proofing[images][${itemId}]"]`);

		const enviraProofing = document.querySelector('.envirabox-stage .envira-proofing');

		// If .envira-proofing exists, remove it
		if(enviraProofing){
			enviraProofing.remove();
		}

		const newEnviraProofing = document.createElement('div');
		newEnviraProofing.classList.add('envira-proofing');

		const form = document.createElement('form');
		form.action = '#';
		form.method = 'post';
		form.classList.add('envira-proofing-form', 'envira-proofing-form-lightbox');

		const fieldDiv = document.createElement('div');
		fieldDiv.classList.add('envira-proofing-field');

		const label = document.createElement('label');
		label.htmlFor = 'envira_proofing_lightbox_add_to_order';
		label.textContent = proofing_add_to_order_label;
		label.classList.add('envira-proofing-checkbox-label');

		const input = document.createElement('input');
		input.id = 'envira_proofing_lightbox_add_to_order';
		input.type = 'checkbox';
		input.name = 'envira_proofing_lightbox[images]';
		input.value = '1';
		input.classList.add('envira-proofing-checkbox');
		input.setAttribute('data-item-id', itemId);
		input.checked = galleryCheckbox.checked;
		input.addEventListener('change', event => {
			EnviraProofing.changeLBCheckbox(event.target);
		});

		fieldDiv.appendChild(input);
		fieldDiv.appendChild(label);


		const fieldsDiv = document.createElement('div');
		fieldsDiv.classList.add('envira-proofing-fields');
		fieldsDiv.style.display = galleryCheckbox.checked ? 'block' : 'none';

		if(quantityEnabled && validSizes){
			for(let prop in sizes){
				const numberElement = document.querySelector(`input[type="number"][name="envira_proofing[quantities][${itemId}][${sizes[prop].name}]"]`);
				const sizeFieldDiv = document.createElement('div');
				sizeFieldDiv.classList.add('envira-proofing-field');

				const sizeLabel = document.createElement('label');
				sizeLabel.htmlFor = `envira_proofing_lightbox_quantity_${sizes[prop].slug}`;
				sizeLabel.textContent = sizes[prop].name;

				const sizeInput = document.createElement('input');
				sizeInput.type = 'number';
				sizeInput.name = `envira_proofing_lightbox[quantities][${sizes[prop].name}]`;
				sizeInput.id = `envira_proofing_lightbox_quantity_${sizes[prop].slug}`;
				sizeInput.setAttribute('data-item-id', itemId);
				sizeInput.setAttribute('data-size', sizes[prop].name);
				sizeInput.min = 1;
				sizeInput.value = numberElement.value;
				sizeInput.classList.add('envira-proofing-number');
				sizeInput.addEventListener('change', event => {
					EnviraProofing.changeLBFieldsSizesQuantityNumber(event.target);
				});

				sizeFieldDiv.appendChild(sizeLabel);
				sizeFieldDiv.appendChild(sizeInput);
				fieldsDiv.appendChild(sizeFieldDiv);
			}
		} else if(quantityEnabled && !sizesEnabled){
			const numberElement = document.querySelector(`input[type="number"][name="envira_proofing[quantities][${itemId}]"]`);
			const quantityFieldDiv = document.createElement('div');
			quantityFieldDiv.classList.add('envira-proofing-field');

			const quantityLabel = document.createElement('label');
			quantityLabel.htmlFor = 'envira_proofing_lightbox_quantity';
			quantityLabel.textContent = 'Quantity'; // TODO: Translate this label

			const quantityInput = document.createElement('input');
			quantityInput.type = 'number';
			quantityInput.name = 'envira_proofing_lightbox[quantities]';
			quantityInput.id = 'envira_proofing_lightbox_quantity';
			quantityInput.min = 0;
			quantityInput.value = numberElement.value;
			quantityInput.setAttribute('data-item-id', itemId);
			quantityInput.classList.add('envira-proofing-number');
			quantityInput.addEventListener('change', event => {
				EnviraProofing.changeLBFieldsQuantitiesNumber(event.target);
			});

			quantityFieldDiv.appendChild(quantityLabel);
			quantityFieldDiv.appendChild(quantityInput);
			fieldsDiv.appendChild(quantityFieldDiv);
		} else if(!quantityEnabled && validSizes){
			for(let prop in sizes){
				const checkboxElement = document.querySelector(`input[type="checkbox"][name="envira_proofing[quantities][${itemId}][${sizes[prop].name}]"]`);
				const elementChecked = checkboxElement.checked;
				const sizeFieldDiv = document.createElement('div');
				sizeFieldDiv.classList.add('envira-proofing-field');


				const sizeLabel = document.createElement('label');
				sizeLabel.htmlFor = `envira_proofing_lightbox_quantity_${sizes[prop].slug}`;
				sizeLabel.textContent = sizes[prop].name;
				sizeLabel.classList.add('envira-proofing-checkbox-label');

				const sizeInput = document.createElement('input');
				sizeInput.type = 'checkbox';
				sizeInput.name = `envira_proofing_lightbox[quantities][${sizes[prop].name}]`;
				sizeInput.id = `envira_proofing_lightbox_quantity_${sizes[prop].slug}`;
				sizeInput.setAttribute('data-item-id', itemId);
				sizeInput.setAttribute('data-size', sizes[prop].name);
				sizeInput.checked = elementChecked;
				sizeInput.value = '1';
				sizeInput.classList.add('envira-proofing-checkbox');
				sizeInput.addEventListener('change', event => {
					EnviraProofing.changeLBFieldsSizesCheckbox(event.target);
				});


				sizeFieldDiv.appendChild(sizeInput);
				sizeFieldDiv.appendChild(sizeLabel);
				fieldsDiv.appendChild(sizeFieldDiv);
			}
		}

		form.appendChild(fieldDiv);
		form.appendChild(fieldsDiv);

		newEnviraProofing.appendChild(form);

		document.querySelector('.envirabox-stage .envirabox-slide--current .envirabox-image-wrap')
			.appendChild(newEnviraProofing);
	});
}

jQuery(document).on('envira_load', () => {
	const form = document.querySelector('form.envira-proofing-form');
	if(!form){
		return;
	}

	EnviraProofing.galleryId = form.dataset.enviraId;

	EnviraProofing.toggleFieldsObserver();
	EnviraProofing.buttonsObservers();
	EnviraProofing.lightboxObserver();
});
