<?php
/**
 * Posttype admin class.
 *
 * @since 1.0.0
 *
 * @package Envira_Proofing
 * @author  Envira Team
 */

namespace Envira\Proofing\Admin;

// Exit if accessed directly.
use Envira\Admin\Notices;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Posttype admin class.
 *
 * @since 1.7.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */
class Posttype {
	/**
	 * Primary class constructor.
	 *
	 * @since 1.7.0
	 */
	public function __construct() {

		// Update post type messages.
		add_filter( 'post_updated_messages', [ $this, 'messages' ] );

		// Custom post type statuses.
		add_action( 'admin_footer-post.php', [ $this, 'append_post_status_list' ] );

		// Force the menu icon to be scaled to proper size (for Retina displays).
		add_action( 'admin_head', [ $this, 'menu_icon' ] );

		// Add the Universal Header.
		add_action( 'in_admin_header', [ $this, 'admin_header' ], 100 );

		add_action( 'add_meta_boxes_envira_proof_order', [ $this, 'add_order_details_meta_box' ] );
		add_action( 'add_meta_boxes_envira_proof_order', [ $this, 'add_image_details_meta_box' ] );
		add_action( 'add_meta_boxes_envira_proof_order', [ $this, 'add_order_history_meta_box' ] );

		add_action( 'admin_footer-edit.php', [ $this, 'custom_status_add_in_quick_edit' ] );
	}

	/**
	 * Append The Post Status List
	 *
	 * @since 2.0.0
	 */
	public function append_post_status_list() {
		global $post;
		$select_completed = '';
		$select_pending   = '';
		$select_active    = '';
		$label            = '';
		if ( 'envira_proof_order' === $post->post_type ) {
			if ( 'proofing-completed' === $post->post_status ) {
				$select_completed = ' selected=\"selected\"';
				$label            = '<span id=\"post-status-display\"> Completed</span>';
			}
			if ( 'proofing-pending' === $post->post_status ) {
				$select_pending = ' selected=\"selected\"';
				$label          = '<span id=\"post-status-display\"> Pending</span>';
			}
			if ( 'proofing-active' === $post->post_status ) {
				$select_active = ' selected=\"selected\"';
				$label         = '<span id=\"post-status-display\"> Active</span>';
			}
			echo '
			<script>
			jQuery(document).ready(function($){
				$("select#post_status").append("<option value=\"proofing-completed\" ' . esc_html( $select_completed ) . '>Completed</option>");
				$("select#post_status").append("<option value=\"proofing-pending\" ' . esc_html( $select_pending ) . '>Pending</option>");
				$("select#post_status").append("<option value=\"proofing-active\" ' . esc_html( $select_active ) . '>Active</option>");
				$(".misc-pub-section #post-status-display").replaceWith("' . wp_kses( $label, envira_proofing_wp_kses_allowed_html() ) . '");
				$("#post_status").on("change", "#post-status-select", function(){
					alert("here");
				});
				$(".save-post-status").on("click", "#post-status-select", function(){
					alert("here");
				});

			});
			</script>
			';
		}
	}

	/**
	 * Quick Edit.
	 *
	 * @since 2.0.0
	 */
	public function custom_status_add_in_quick_edit() {
		echo "<script>
		jQuery(document).ready( function() {
		    jQuery( 'select[name=\"_status\"] option[value=\"publish\"]' ).remove();
		    jQuery( 'select[name=\"_status\"] option[value=\"pending\"]' ).remove();
		    jQuery( 'select[name=\"_status\"] option[value=\"draft\"]' ).remove();
		    jQuery( 'select[name=\"_status\"]' ).append( '<option value=\"proofing-completed\">Completed</option>' );
		    jQuery( 'select[name=\"_status\"]' ).append( '<option value=\"proofing-pending\">Pending</option>' );
		    jQuery( 'select[name=\"_status\"]' ).append( '<option value=\"proofing-active\">Active</option>' );
		});
		</script>";
	}

	/**
	 * Register the custom statuses for post pages.
	 *
	 * @since 2.0.0
	 */
	public function custom_status_add_in_post_page() {
		echo "<script>
		jQuery(document).ready( function() {
		    jQuery( 'select[name=\"post_status\"]' ).append( '<option value=\"completed\">Completed</option>' );
		    jQuery( 'select[name=\"post_status\"]' ).append( '<option value=\"pending\">Pending</option>' );
		    jQuery( 'select[name=\"post_status\"]' ).append( '<option value=\"active\">Active</option>' );
		});
		</script>";
	}


	/**
	 * Register the custom statuses for meta boxes.
	 *
	 * @since 2.0.0
	 */
	public function add_order_details_meta_box() {

		add_meta_box(
			'order_details_metabox',
			'Order Details',
			[ $this, 'order_details_metabox' ],
			'envira_proof_order',
			'normal', // instead of side.
			'default'
		);
	}

	/**
	 * Register the image details meta box.
	 *
	 * @since 2.0.0
	 */
	public function add_image_details_meta_box() {
	  	// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ! isset( $_GET['post'] ) ) {
			return;
		}
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$proofing_images = get_post_meta( intval( $_GET['post'] ), 'proofing_images', true );

		if ( ! ( is_array( $proofing_images ) || $proofing_images instanceof \Countable ) ) {
			$proofing_images = [];
		}

		add_meta_box(
			'image_details_metabox',
			'Images Ordered (' . count( $proofing_images ) . ')',
			[ $this, 'image_details_metabox' ],
			'envira_proof_order',
			'normal', // instead of side.
			'default'
		);
	}

	/**
	 * Register the order details metabox.
	 *
	 * @since 2.0.0
	 */
	public function order_details_metabox() {
		// phpcs:disable WordPress.Security.NonceVerification.Recommended

		if ( ! isset( $_GET['post'] ) ) {
			return;
		}

		$post_id     = intval( $_GET['post'] );
		$order       = get_post( $post_id );
		$new_gallery = isset( $_GET['new_gallery'] ) ? intval( $_GET['new_gallery'] ) : 0;

	    // phpcs:enable WordPress.Security.NonceVerification.Recommended

		$customer_name  = get_post_meta( $post_id, 'proofing_name', true );
		$customer_notes = get_post_meta( $post_id, 'proofing_notes', true );
		$customer_email = get_post_meta( $post_id, 'proofing_email', true );
		$gallery_id     = get_post_meta( $post_id, 'gallery_id', true );

		$default_gallery_name = apply_filters( 'envira_proofing_create_gallery_name', 'Gallery For ' . $order->post_title . ' (' . $customer_name . ')', $order, $gallery_id, $customer_name, $customer_email );

		if ( $new_gallery ) {
			/* translators: %s: URL */
			$message = sprintf( __( '<strong>Envira Proofing</strong>: New Gallery <a href="%s">has been created</a>.', 'envira-gallery' ), get_permalink( $new_gallery ) );
			$notices = new Notices();
			$notices->display_inline_notice( 'gallery-created', false, $message, 'warning', false, false, false, false );
		}
		?>

			<table class="form-table" style="margin-bottom: 0;">
				<tbody>
					<tr id="envira-proofing-order-details-customer-name">
						<th scope="row">
							<label for="envira-proofing-order-details-customer-name">Customer Name</label>
						</th>
						<td>
							<input id="envira-proofing-order-details-customer-name" type="text" name="_envira_gallery[proofing_name]" value="<?php echo esc_html( $customer_name ); ?>">
						</td>
					</tr>
					<tr id="envira-proofing-order-details-customer-email">
						<th scope="row">
							<label for="envira-proofing-order-details-customer-email">Customer Email</label>
						</th>
						<td>

							<input id="envira-proofing-order-details-customer-email" type="text" name="_envira_gallery[proofing_email]" value="<?php echo ( isset( $customer_email ) && is_email( esc_html( $customer_email ) ) ? esc_html( $customer_email ) : '' ); ?>">
							<a href="<?php echo esc_url( wp_nonce_url( admin_url( 'post.php?post=' . $post_id . '&action=edit&resend_email=1' ), 'envira_gallery_proofing_resend_confirmation' ) ); ?>" class="button button-primary"><?php esc_html_e( 'Resend Confirmation Email', 'envira-proofing' ); ?></a>

						</td>
					</tr>
					<tr id="envira-proofing-order-details-gallery-id">
						<th scope="row">
							<label for="envira-proofing-order-details-gallery-id">Gallery</label>
						</th>
						<td>
							<?php

								$gallery = get_post( $gallery_id );
								echo ( isset( $gallery_id ) && ! empty( $gallery ) ? '<a target="_blank" href="' . esc_url( get_edit_post_link( $gallery_id ) ) . '">' . esc_html( $gallery->post_title ) . '</a>' : 'Unknown' );
							?>
						</td>
					</tr>
					<tr id="envira-proofing-order-details-customer-notes">
						<th scope="row">
							<label for="envira-proofing-order-details-customer-notes">Customer Notes</label>
						</th>
						<td>
							<?php
								echo '<textarea style="width: 95%; min-height: 50px;" rows="10">';
								echo esc_html( $customer_notes );
								echo '</textarea>';
							?>
						</td>
					</tr>
					<tr id="envira-proofing-order-details-create-gallery">
						<th scope="row">
							<label for="envira-proofing-order-details-customer-email">Create Gallery</label>
						</th>
						<td>
							Name:
							<input id="envira-proofing-order-details-customer-email" type="text" name="_envira_gallery[proofing_email]" value="<?php echo esc_html( $default_gallery_name ); ?>">
							<a href="<?php echo esc_url( wp_nonce_url( admin_url( 'post.php?post=' . $post_id . '&action=edit&create_gallery=1' ), 'envira_gallery_create_gallery_confirmation' ) ); ?>" class="button button-primary"><?php esc_html_e( 'Create Gallery', 'envira-proofing' ); ?></a>

						</td>
					</tr>
				</tbody>
			</table>


		<?php
	}

	/**
	 * Register the image details metabox.
	 *
	 * @since 2.0.0
	 */
	public function image_details_metabox() {
	  	// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$post_id = isset( $_GET['post'] ) ? intval( $_GET['post'] ) : false;

		$customer_name   = get_post_meta( $post_id, 'proofing_name', true );
		$customer_email  = get_post_meta( $post_id, 'proofing_email', true );
		$gallery_id      = get_post_meta( $post_id, 'gallery_id', true );
		$gallery_link    = get_edit_post_link( $gallery_id );
		$gallery_data    = envira_get_gallery( $gallery_id );
		$proofing_images = get_post_meta( $post_id, 'proofing_images', true );
		$total_count     = 0;

		if ( empty( $proofing_images ) ) {

			?>

		<p>No images have been added to this order.</p>

			<?php

			return;

		}

		?>

			<table class="form-table" style="margin-bottom: 0;">
				<thead>
					<th colspan="2" style="padding-bottom: 0; padding-top: 0;">Image</th>
					<th style="padding-right: 0; padding-bottom: 0; padding-top: 0;">
						<table style="width: 100%;">
							<thead>
								<th style="padding: 0; width: 50%;">Size</th>
								<th style="padding: 0; text-align: center;">Amount</th>
							</thead>
						</table>
					</th>
				</thead>
				<tbody>
					<?php
					foreach ( $proofing_images as $proofing_image_id => $proofing_image_details ) {

							$proofing_image       = get_post( $proofing_image_id );
							$proofing_image_title = ( ! empty( $gallery_data['gallery'][ $proofing_image_id ]['title'] ) ) ? esc_html( $gallery_data['gallery'][ $proofing_image_id ]['title'] ) : $proofing_image->post_title;

							$proofing_sizes = ( is_array( $proofing_image_details ) ) ? $proofing_image_details : false;

						?>
					<tr id="">
						<td style="text-align: left; width: 50px; padding-left: 0;">
							<a rel="noopener" target="_blank" href="<?php echo esc_url( $gallery_link ); ?>">
							<?php echo wp_get_attachment_image( $proofing_image_id, [ '50', '50' ], '', [ 'class' => 'img-responsive' ] ); ?>
						</a>
						<td style="padding: 0;">
							<a rel="noopener" target="_blank" href="<?php echo esc_url( $gallery_link ); ?>"><?php echo esc_html( $proofing_image_title ); ?></a>
						</td>
						<td style="padding: 0;" colspan="2">
						<?php

						if ( ! empty( $proofing_sizes ) ) {
							foreach ( $proofing_sizes as $size => $count ) {
									$total_count += intval( $count );
								?>
									<table style="width: 100%;">
										<tr id="">
											<td style="padding: 0; width: 50%;"><?php echo esc_html( $size ); ?></td>
											<td style="padding: 0; text-align:center;"><?php echo intval( $count ); ?></td>
										</tr>
									</table>
									<?php
							}
						}
						?>
						</td>
					</tr>

					<?php } ?>

					<tr>
						<td colspan="4" style="padding: 10px 0;">
							<hr/>
						</td>
					</tr>
					<tr id="">
						<td style="padding: 0;"></td>
						<td style="padding: 0;"></td>
						<td style="padding: 0;">
						<table style="width: 100%;">
							<thead>
								<th style="padding: 0; width: 50%;">TOTAL</th>
								<th style="padding: 0; text-align: center;"><?php echo esc_html( $total_count ); ?></th>
							</thead>
						</table>
						</td>

					</tr>

				</tbody>
			</table>

		<?php
	}

	/**
	 * Register the order history metabox.
	 *
	 * @since 2.0.0
	 */
	public function order_history_metabox() {
	  	// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$post_id = isset( $_GET['post'] ) ? intval( $_GET['post'] ) : false;

		$order_history = get_post_meta( $post_id, '_envira_proofing_order_history', true );

		if ( empty( $order_history ) ) {
			echo '<p>Order History Not Available.</p>';
			return;
		}

		// reverse order so it's most recent at top.
		$order_history = array_reverse( $order_history, true );

		?>

			<table class="form-table" style="margin-bottom: 0; width: 95%;">
				<thead>
					<th style="padding-top: 0; text-align: left; width: 20%">Date</th>
					<th style="padding-top: 0; text-align: left; width: 10%">User</th>
					<th style="padding-top: 0; text-align: left; width: 50%">Action</th>
				</thead>
				<tbody>
					<?php
					foreach ( $order_history as $order_history_date => $order_history_item ) {

							$order_history_date = apply_filters( 'envira_gallery_proofing_messages', gmdate( 'F jS, Y g:ia', $order_history_date ), $post_id, $order_history, $order_history_date, $order_history_item );

						if ( false === intval( $order_history_item['user_id'] ) ) {
								$user = false;
						} else {
								$user_id = intval( $order_history_item['user_id'] );
								// check and see if this is a user registered.
								$user = get_user_by( 'ID', $user_id );
						}
						?>
					<tr id="">
						<td style="padding-bottom: 0; padding-top: 0; padding-left: 0; text-align: left; width: 20%;"><?php echo esc_html( $order_history_date ); ?></td>
						<td style="padding-bottom: 0; padding-top: 0; text-align: left; width: 10%;"><?php echo ( ! empty( $user->data ) ) ? '<a href="' . esc_url( get_edit_user_link( $user->data->ID ) ) . '">' . esc_html( $user->data->user_login ) : '</a>'; ?></td>
						<td style="padding-bottom: 0; padding-top: 0; text-align: left; width: 50%;"><?php echo esc_html( $order_history_item['message'] ); ?></td>
					</tr>
					<?php } ?>
				</tbody>
			</table>


		<?php
	}

	/**
	 * Adds the order history meta box.
	 *
	 * @since 2.0.0
	 */
	public function add_order_history_meta_box() {
		add_meta_box(
			'order_history_metabox',
			'Order History',
			[ $this, 'order_history_metabox' ],
			'envira_proof_order',
			'normal', // instead of side.
			'default'
		);
	}


	/**
	 * Outputs the Envira Gallery Header.
	 *
	 * @since 1.5.0
	 */
	public function admin_header() {
		// Get the current screen, and check whether we're viewing the Envira or Envira Album Post Types.
		$screen = get_current_screen();
		if ( 'envira_proof_order' !== $screen->post_type ) {
			return;
		}
		// If here, we're on an Envira Gallery or Album screen, so output the header.
		envira_load_admin_partial(
			'header',
			[
				'logo' => plugins_url( 'assets/images/envira-logo-color.svg', ENVIRA_FILE ),
			]
		);
	}

	/**
	 * Contextualizes the post updated messages.
	 *
	 * @since 1.7.0
	 *
	 * @global object $post    The current post object.
	 * @param array $messages  Array of default post updated messages.
	 * @return array $messages Amended array of post updated messages.
	 */
	public function messages( $messages ) {

		global $post;
	  	// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$revision_id = isset( $_GET['revision'] ) ? intval( $_GET['revision'] ) : false;

		// Contextualize the messages.
		$envira_messages = [
			0  => '',
			1  => __( 'Order updated.', 'envira-gallery' ),
			2  => __( 'Order custom field updated.', 'envira-gallery' ),
			3  => __( 'Order custom field deleted.', 'envira-gallery' ),
			4  => __( 'Order updated.', 'envira-gallery' ),
			// translators: %s: revision id.
			5  => $revision_id ? sprintf( __( 'Order restored to revision from %s.', 'envira-gallery' ), wp_post_revision_title( $revision_id, false ) ) : false,
			6  => __( 'Order published.', 'envira-gallery' ),
			7  => __( 'Order saved.', 'envira-gallery' ),
			8  => __( 'Order submitted.', 'envira-gallery' ),
			/* translators: %s: date time */
			9  => sprintf( __( 'Order scheduled for: <strong>%1$s</strong>.', 'envira-gallery' ), date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ) ),
			10 => __( 'Order draft updated.', 'envira-gallery' ),
		];

		$messages['envira_proof_order'] = apply_filters( 'envira_gallery_proofing_messages', $envira_messages );

		return $messages;
	}

	/**
	 * Forces the Envira menu icon width/height for Retina devices.
	 *
	 * @since 1.7.0
	 */
	public function menu_icon() {

		?>
		<style type="text/css">#menu-posts-envira_proof_order .wp-menu-image img { width: 16px; height: 16px; }</style>
		<?php
	}
}
