<?php
/**
 * Posttype class.
 *
 * @since 2.0.0
 *
 * @package Envira Gallery
 * @subpackage Envira Orders
 * @author Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Proofing\Frontend;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Orders Post Type Class.
 *
 * @since 2.0.0
 */
class Posttype {

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		$envira_whitelabel = apply_filters( 'envira_whitelabel', false );

		// Build the labels for the post type.
		$labels = apply_filters(
			'envira_orders_post_type_labels',
			[
				'name'               => $envira_whitelabel ? apply_filters( 'envira_order_whitelabel_name_plural', false ) : __( 'Envira Proofing Orders', 'envira-proofing' ),
				'name_admin_bar'     => $envira_whitelabel ? apply_filters( 'envira_order_whitelabel_name_plural', false ) : __( 'Envira Proofing Orders', 'envira-proofing' ),
				'singular_name'      => $envira_whitelabel ? apply_filters( 'envira_order_whitelabel_name', false ) : __( 'Envira Proofing Order', 'envira-proofing' ),
				'add_new'            => __( 'Add New', 'envira-orders' ),
				'add_new_item'       => $envira_whitelabel ? __( 'Add New Order', 'envira-gallery' ) : __( 'Add New Envira Proofing Order', 'envira-proofing' ),
				'edit_item'          => $envira_whitelabel ? __( 'Edit Order', 'envira-gallery' ) : __( 'Edit Envira Proofing Order', 'envira-proofing' ),
				'new_item'           => $envira_whitelabel ? __( 'New Order', 'envira-gallery' ) : __( 'New Envira Proofing Order', 'envira-proofing' ),
				'view_item'          => $envira_whitelabel ? __( 'View Order', 'envira-gallery' ) : __( 'View Envira Proofing Order', 'envira-proofing' ),
				'search_items'       => $envira_whitelabel ? __( 'Search Orders', 'envira-gallery' ) : __( 'Search Envira Proofing Orders', 'envira-proofing' ),
				'not_found'          => $envira_whitelabel ? __( 'No orders found.', 'envira-gallery' ) : __( 'No Envira Proofing orders found.', 'envira-proofing' ),
				'not_found_in_trash' => $envira_whitelabel ? __( 'No orders found in trash.', 'envira-gallery' ) : __( 'No Envira Proofing orders found in trash.', 'envira-proofing' ),
				'parent_item_colon'  => '',
				'menu_name'          => __( 'Orders', 'envira-proofing' ),
			]
		);

		// Build out the post type arguments.
		$args = apply_filters(
			'envira_proofing_orders_post_type_args',
			[
				'labels'              => $labels,
				'public'              => false,
				'exclude_from_search' => false,
				'show_ui'             => true,
				'show_in_menu'        => true,
				'show_in_admin_bar'   => false,
				'rewrite'             => false,
				'query_var'           => false,
				'has_archive'         => false,
				'show_in_nav_menus'   => false,
				'menu_position'       => apply_filters( 'envira_gallery_post_type_menu_position', 250 ),
				'menu_icon'           => plugins_url( 'assets/css/images/menu-icon@2x.png', ENVIRA_FILE ),
				'supports'            => [ 'title' ],
				'capabilities'        => [
					// Meta caps.
					'edit_post'              => 'edit_envira_proof',
					'read_post'              => 'read_envira_proof',
					'delete_post'            => 'delete_envira_proof',

					// Primitive caps outside map_meta_cap().
					'edit_others_posts'      => 'edit_other_envira_proofs',
					'publish_posts'          => 'publish_envira_proofs',
					'read_private_posts'     => 'read_private_envira_proofs',

					// Primitive caps used within map_meta_cap().
					'read'                   => 'read',
					'delete_posts'           => 'delete_envira_proofs',
					'delete_private_posts'   => 'delete_private_envira_proofs',
					'delete_published_posts' => 'delete_published_envira_proofs',
					'delete_others_posts'    => 'delete_others_envira_proofs',
					'edit_private_posts'     => 'edit_private_envira_proofs',
					'edit_published_posts'   => 'edit_published_envira_proofs',
					'edit_posts'             => 'create_envira_proofs',
				],
				'map_meta_cap'        => true,

			]
		);

		// Register the post type with WordPress.
		register_post_type( 'envira_proof_order', $args );

		// Custom Post Statuses.
		register_post_status(
			'proofing-completed',
			[
				'label'                     => _x( 'Completed', 'envira-gallery-proofing' ),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'post_type'                 => [ 'envira_proof_order' ],
				// Translators: Complete.
				'label_count'               => _n_noop( 'Completed <span class="count">(%s)</span>', 'Completed <span class="count">(%s)</span>' ),
			]
		);

		register_post_status(
			'proofing-pending',
			[
				'label'                     => _x( 'Pending', 'envira-gallery-proofing' ),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'post_type'                 => [ 'envira_proof_order' ],
				// Translators: Complete.
				'label_count'               => _n_noop( 'Pending <span class="count">(%s)</span>', 'Pending <span class="count">(%s)</span>' ),
			]
		);

		register_post_status(
			'proofing-active',
			[
				'label'                     => _x( 'Active', 'envira-gallery-proofing' ),
				'public'                    => true,
				'exclude_from_search'       => false,
				'show_in_admin_all_list'    => true,
				'show_in_admin_status_list' => true,
				'post_type'                 => [ 'envira_proof_order' ],
				// Translators: Complete.
				'label_count'               => _n_noop( 'Active <span class="count">(%s)</span>', 'Active <span class="count">(%s)</span>' ),
			]
		);
	}
}
