<?php
/**
 * Proofing Functions
 *
 * @package Envira Proofing
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Helper Method to flust proofing caches
 *
 * @access public
 * @param int    $post_id proofing post id.
 * @param string $slug (default: '').
 * @return void
 */
function envira_flush_proofing_caches( $post_id, $slug = '' ) {

	// Delete known proofing caches.
	delete_transient( '_eg_cache_' . $post_id );
	delete_transient( '_ea_cache_all' );
	delete_transient( '_eg_fragment_proofings_' . $post_id );

	// Possibly delete slug gallery cache if available.
	if ( ! empty( $slug ) ) {
		delete_transient( '_eg_cache_' . $slug );
	}

	// Run a hook for Addons to access.
	do_action( 'envira_proofings_flush_caches', $post_id, $slug );
}

/**
 * Helper method to get proofing config defaults
 *
 * @access public
 * @param int $key The setting we need.
 * @return array
 */
function envira_proofing_get_global_config_defaults( $key = false ) {

	$defaults = [];

	// Lightbox.
	$defaults['global_proofing_lightbox'] = 0;

	// Proofing.
	$defaults['global_proofing']                         = 0;
	$defaults['global_proofing_multiple_enabled']        = 0;
	$defaults['global_proofing_edit']                    = 0;
	$defaults['global_proofing_delete']                  = 0;
	$defaults['global_proofing_name_enabled']            = 0;
	$defaults['global_proofing_hide_gallery']            = 1;
	$defaults['global_proofing_multiple_label']          = __( 'Please enter your email address to begin the selection process.', 'envira-proofing' );
	$defaults['global_proofing_quantity_enabled']        = 0;
	$defaults['global_proofing_size_enabled']            = 0;
	$defaults['global_proofing_sizes']                   = [];
	$defaults['global_proofing_add_to_order_label']      = __( 'Add to Order', 'envira-proofing' );
	$defaults['global_proofing_save_button_label']       = __( 'Save', 'envira-proofing' );
	$defaults['global_proofing_edit_button_label']       = __( 'Edit Order', 'envira-proofing' );
	$defaults['global_proofing_delete_button_label']     = __( 'Delete Order', 'envira-proofing' );
	$defaults['global_proofing_notes_placeholder_text']  = __( 'Order Notes (Optional)', 'envira-proofing' );
	$defaults['global_proofing_save_notes_button_label'] = __( 'Save Order Notes', 'envira-proofing' );
	$defaults['global_proofing_submit_button_label']     = __( 'Submit Order', 'envira-proofing' );

	$defaults['global_proofing_submitted_message']  = __( 'Thanks for submitting your order. We will be in touch.', 'envira-proofing' );
	$defaults['global_proofing_saved_message']      = __( 'Thanks for saving your order. Don\'t forget to submit it!', 'envira-proofing' );
	$defaults['global_proofing_email']              = '';
	$defaults['global_proofing_email_box_position'] = '';
	$defaults['global_proofing_email_subject']      = __( 'Envira Proofing: New Order for Gallery: {title}', 'envira-proofing' );
	$defaults['global_proofing_email_message']      = __( 'A new order has been submitted by {order_email} for Gallery {title}', 'envira-proofing' ) . "\n\n" . __( 'View the order at {url}', 'envira-proofing' );

	$defaults['global_proofing_admin_email_notification']   = 1;
	$defaults['global_proofing_admin_email']                = get_bloginfo( 'admin_email' );
	$defaults['global_proofing_admin_email_subject']        = __( 'Envira Proofing: New Order for Gallery: {title}', 'envira-proofing' );
	$defaults['global_proofing_admin_email_subject_save']   = __( 'Envira Proofing: New Order Saved for Gallery: {title}', 'envira-proofing' );
	$defaults['global_proofing_admin_email_subject_submit'] = __( 'Envira Proofing: New Order Submitted for Gallery: {title}', 'envira-proofing' );
	$defaults['global_proofing_admin_email_message']        = __( 'A new order has been submitted by {order_email} for Gallery {title}', 'envira-proofing' ) . "\n\n" . __( 'View the order at {url}', 'envira-proofing' );

	$defaults['global_proofing_customer_email_subject_save'] = __( 'Thank you for saving your order!', 'envira-proofing' );
	$defaults['global_proofing_customer_email_message_save'] = __( 'Thank you for saving your order!', 'envira-proofing' ) . "\n\n" . __( 'View the order at {url}', 'envira-proofing' );

	$defaults['global_proofing_customer_email_subject_submit'] = __( 'Thank you for submitting your order!', 'envira-proofing' );
	$defaults['global_proofing_customer_email_message_submit'] = __( 'Thank you for submitting your order!', 'envira-proofing' ) . "\n\n" . __( 'View the order at {url}', 'envira-proofing' );

	$defaults['proofing_email_logo']            = '';
	$defaults['proofing_email_mailing_address'] = get_bloginfo( 'name' ) . "\n" .
												get_bloginfo( 'url' ) . "\n" .
												__( '1234 Sunny Road', 'envira-proofing' ) . "\n" .
												__( 'Sunnyville, CA 12345', 'envira-proofing' );

	$defaults['proofing_override_globals'] = 0;
	$defaults['warning_message_disabled']  = 0;

	if ( false !== $key && isset( $defaults[ $key ] ) ) {
		if ( ! is_array( $defaults[ $key ] ) ) {
			$defaults = (string) $defaults[ $key ];
		} else {
			$defaults = $defaults[ $key ];
		}
	}

	if ( false !== $key && ! empty( $defaults[ 'global_' . $key ] ) ) {
		if ( ! is_array( $defaults[ 'global_' . $key ] ) ) {
			$defaults = (string) $defaults[ 'global_' . $key ];
		} else {
			$defaults = $defaults[ 'global_' . $key ];
		}
	}

	return apply_filters( 'envira_proofings_defaults', $defaults, $key );
}


/**
 * Helper method for getting a setting's value. Falls back to the default
 * setting value if none exists in the options table.
 *
 * @since 1.7.0
 *
 * @param string $key   The setting key to retrieve.
 * @return string       Key value on success, false on failure.
 */
function envira_proofing_get_global_setting( $key ) {

	// Prefix the key.
	$prefixed_key = 'envira_gallery_' . $key;
	$option_value = get_option( $prefixed_key );

	// If no value exists, fallback to the default.
	if ( ! isset( $option_value ) || false === $option_value ) {
		$option_value = envira_proofing_get_global_config_defaults( $key );
	}

	// Allow devs to filter.
	$value = apply_filters( 'envira_proofing_get_global_setting', $option_value, $key, $prefixed_key );

	return $value;
}

/**
 * Helper method for getting a setting's value given a gallery id.
 *
 * @since 1.7.0
 *
 * @param string  $key   The setting key to retrieve.
 * @param integer $gallery_id The gallery ID.
 * @param boolean $use_global Do we use global.
 * @return string       Key value on success, false on failure.
 */
function envira_proofing_get_setting_from_gallery( $key, $gallery_id = false, $use_global = false ) {

	$value = '';

	// If a gallery_id is supplied, check the gallery specific setting for the key first.
	// Note that a valid value can be false, so if we can't find something it's null.
	if ( false !== $gallery_id ) {
		$data = envira_get_gallery_data( intval( $gallery_id ) );
	}

	$overrideable_settings_keys = [ 'proofing_customer_email_message_submit' ];

	if ( in_array( $key, $overrideable_settings_keys, true ) && 0 === intval( $data['config']['proofing_override_globals'] ) ) {
		// if we are looking for a setting in a gallery that is overrideable but the checkbox isn't true to override it, pull from the global proofing settings.
		$value = envira_proofing_get_global_setting( 'global_' . $key );
		return $value;
	}

	// To-do: The following needs to be refactored.

	if ( 'proofing_customer_email_subject_submit' === $key && 1 === intval( $data['config']['proofing_override_globals'] ) ) { // if there is an override checked in gallery.
		$legacy_key = 'proofing_email_subject'; // use the setting from the gallery instead, this is what it was called in version 1.x.
		$value      = isset( $data['config'][ $legacy_key ] ) ? $data['config'][ $legacy_key ] : false;
	} elseif ( 'proofing_customer_email_message_submit' === $key && 1 === intval( $data['config']['proofing_override_globals'] ) ) { // if there is an override checked in gallery.
		$legacy_key = 'proofing_email_message'; // use the setting from the gallery instead, this is what it was called in version 1.x.
		$value      = isset( $data['config'][ $legacy_key ] ) ? $data['config'][ $legacy_key ] : false;
	} elseif ( 'proofing_submitted_message' === $key && 1 === intval( $data['config']['proofing_override_globals'] ) ) { // if there is an override checked in gallery.
		$legacy_key = 'proofing_submitted_message'; // use the setting from the gallery instead, this is what it was called in version 1.x.
		$value      = isset( $data['config'] [ $legacy_key ] ) ? $data['config'][ $legacy_key ] : false;
	} elseif ( 'proofing_submitted_message' === $key && 0 === intval( $data['config']['proofing_override_globals'] ) ) { // if there is an override checked in gallery.
		$value = envira_proofing_get_global_setting( 'global_proofing_submitted_message' );
	} else {
		$value = isset( $data['config'][ $key ] ) ? $data['config'][ $key ] : false;
	}

	// If no value exists, fallback to the default.
	if ( false === $value && true === $use_global ) {
		$value = envira_proofing_get_global_setting( $key );
	}

	// Allow devs to filter.
	$value = apply_filters( 'envira_proofing_get_setting_from_gallery', $value, $key, $gallery_id );

	return $value;
}
/**
 * Helper method for getting a setting's value.
 *
 * @since 1.7.0
 *
 * @param string  $key   The setting key to retrieve.
 * @param integer $gallery_id The gallery ID.
 * @return string       Key value on success, false on failure.
 */
function envira_proofing_get_setting( $key, $gallery_id = false ) {

	$key = trim( $key );

	if ( false !== $gallery_id ) {
		return envira_proofing_get_setting_from_gallery( $key, $gallery_id );
	} else {
		return envira_proofing_get_global_setting( $key );
	}
}

/**
 * Helper method for retrieving size orientations.
 *
 * @since 1.0.0
 *
 * @return array Array of column data.
 */
function envira_proofing_get_size_orientations() {

	$orientations = [
		[
			'value' => '',
			'name'  => __( 'Landscape and Portrait', 'envira-proofing' ),
		],
		[
			'value' => 'landscape',
			'name'  => __( 'Landscape', 'envira-proofing' ),
		],
		[
			'value' => 'portrait',
			'name'  => __( 'Portrait', 'envira-proofing' ),
		],
	];

	return apply_filters( 'envira_proofing_size_orientations', $orientations );
}

/**
 * Helper method for retrieving email box positions.
 *
 * @since 1.0.0
 *
 * @return array Array of column data.
 */
function envira_proofing_get_proofing_email_box_positions() {

	$orientations = [
		[
			'value' => '',
			'name'  => __( 'Above', 'envira-proofing' ),
		],
		[
			'value' => 'below',
			'name'  => __( 'Below', 'envira-proofing' ),
		],
	];

	return apply_filters( 'envira_proofing_email_box_positions', $orientations );
}

/**
 * Helper method for retrieving allowed html.
 */
function envira_proofing_wp_kses_allowed_html() {
	return [
		'a'      => [
			'href'                => [],
			'target'              => [],
			'class'               => [],
			'title'               => [],
			'data-status'         => [],
			'data-envira-tooltip' => [],
			'data-id'             => [],
		],
		'br'     => [],
		'img'    => [
			'src'   => [],
			'class' => [],
			'alt'   => [],
		],
		'h1'     => [
			'class' => [],
		],
		'h2'     => [
			'class' => [],
		],
		'h3'     => [
			'class' => [],
		],
		'h4'     => [
			'class' => [],
		],
		'h5'     => [
			'class' => [],
		],
		'h6'     => [
			'class' => [],
		],
		'div'    => [
			'class' => [],
		],
		'li'     => [
			'id'                              => [],
			'class'                           => [],
			'data-envira-gallery-image'       => [],
			'data-envira-gallery-image-model' => [],
		],
		'em'     => [],
		'span'   => [
			'class' => [],
		],
		'strong' => [],
	];
}
