/**
 * Envira Tags
 *
 * @var EnviraTags -- Defined in localize script.
 */

EnviraTags.init = () => {
	EnviraTags.clickObservers();
};

EnviraTags.clickObservers = () => {
	document.querySelectorAll('.envira-tags-filter-link').forEach((tagLink) => {
		tagLink.addEventListener('click', (event) => {
			event.preventDefault();
			event.stopPropagation();

			if(tagLink.classList.contains('envira-tags-filter-active')){
				return;
			}

			// Add 'envira-tags-filter-active' class to the clicked tag.
			const filter = tagLink.closest('.envira-tags-filter-list');
			const scrollTo = tagLink.getAttribute('href') !== '#';
			const active = filter.querySelector('.envira-tags-filter-active');
			if(active){
				active.classList.remove('envira-tags-filter-active');
			}

			tagLink.classList.add('envira-tags-filter-active');

			const tag = tagLink.dataset.enviraFilter;
			const galleryWrapper = tagLink.closest('.envira-gallery-wrap');

			// Use a different name for the parameter to avoid shadowing
			EnviraTags.filter(tag, galleryWrapper, scrollTo);
		});
	});
};

EnviraTags.filter = (tag, galleryWrapper, scrollTo) => {
	const is_album = galleryWrapper.classList.contains('envira-album-wrap');
	const public_class = is_album ? '.envira-album-public' : '.envira-gallery-public';
	const gallery = galleryWrapper.querySelector(public_class);
	gallery.innerHTML = '<div class="envira-loader"><div></div><div></div><div></div><div></div></div>';

	const parsed_attrs = JSON.parse(gallery.dataset.parsedAttrs);
	if(galleryWrapper.classList.contains('envira-layout-bnb--overlay')){
		parsed_attrs.bnb_overlay = '1';
	}

	jQuery.ajax(EnviraTags.endpoint, {
		type: 'POST',
		data: {
			// nonce: EnviraTags.nonce,
			parsed_attrs: JSON.stringify(parsed_attrs),
			"envira-tag": tag.replace('.envira-tag-', '').replace('.envira-category-', ''),
			type: is_album ? 'album' : 'gallery',
		}
	}).done((response) => {
		if ('' === response) {
			gallery.innerHTML = EnviraTags.noImages;
			return;
		}

		const tempGallery = document.createElement('div');
		tempGallery.innerHTML = response;

		const tempWrapper = tempGallery.querySelector('.envira-gallery-wrap');

		// Replace response filter with original one to not lose events and states.
		const originalFilter = galleryWrapper.querySelector('.envira-tags-filter-list');
		tempWrapper.querySelector('.envira-tags-filter-list').replaceWith(originalFilter);

		// Remove all children from the original gallery.
		Array.from(galleryWrapper.children).forEach((element, index) => {
			element.remove();
		});

		// Append all children from the temp gallery.
		galleryWrapper.append(...Array.from(tempWrapper.children));

		const newGallery = galleryWrapper.querySelector(public_class);

		jQuery(document).trigger('envira_load');

		if(newGallery.classList.contains('enviratope')){
			jQuery(newGallery).enviratope('reloadItems');
		}

		if(scrollTo){
			setTimeout(() => {
				galleryWrapper.scrollIntoView({
					behavior: 'smooth',
					block: 'start',
				});
			});
		}
	}).fail(() => {
		gallery.innerHTML = EnviraTags.errorMsg;
	});
};
