<?php
/**
 * BnB Overlay.
 *
 * @package Envira
 */

use Envira\Frontend\Shortcode;
use Envira\Albums\Frontend\Shortcode as Albums_Shortcode;

/**
 * BnB Overlay REST Methods
 */
class Tags_REST extends WP_REST_Controller {

	/**
	 * Registers the routes for the objects of the controller.
	 *
	 * @see register_rest_route()
	 */
	public function __construct() {
		$this->namespace = 'envira-public/v1';
		$this->rest_base = 'template';
	}

	/**
	 * Registers the routes for the objects of the controller.
	 */
	public function hooks() {
		add_action( 'rest_api_init', [ $this, 'register_routes' ] );
	}

	/**
	 * Registers the routes for the objects of the controller.
	 */
	public function register_routes() {
		register_rest_route(
			$this->namespace,
			$this->rest_base,
			[
				[
					'methods'             => 'POST',
					'callback'            => [ $this, 'get_template' ],
					'args'                => $this->get_template_args(),
					'permission_callback' => '__return_true',
				],
				'schema' => [ $this, 'get_public_item_schema' ],
			]
		);
	}

	/**
	 * Get Template Args
	 *
	 * @return array[]
	 */
	public function get_template_args() {
		return [
			'parsed_attrs' => [
				'required'          => true,
				'type'              => 'string',
				'sanitize_callback' => 'sanitize_text_field',
				'validate_callback' => 'rest_validate_request_arg',
			],
			'envira-tag'   => [
				'required'          => true,
				'type'              => 'string',
				'sanitize_callback' => 'sanitize_text_field',
				'validate_callback' => 'rest_validate_request_arg',
			],
			'type'         => [
				'required'          => false,
				'type'              => 'string',
				'sanitize_callback' => 'sanitize_text_field',
				'validate_callback' => 'rest_validate_request_arg',
			],
		];
	}

	/**
	 * Get Template HTML.
	 *
	 * @param WP_REST_Request $request Request.
	 *
	 * @return string
	 */
	public function get_template( $request ) {
		$parsed_attrs = json_decode( $request->get_param( 'parsed_attrs' ), true );
		$type         = ! empty( $request->get_param( 'type' ) ) ? $request->get_param( 'type' ) : 'gallery';

		if ( ! in_array( $type, [ 'gallery', 'album' ], true ) ) {
			return '';
		}

		$_SERVER['REQUEST_URI'] = $request->get_header( 'referer' );
		$parsed_attrs['cache']  = false;

		if ( 'album' === $type ) {
			return ( new Albums_Shortcode() )->shortcode( $parsed_attrs );
		}

		return ( new Shortcode() )->shortcode( $parsed_attrs );
	}
}
