/**
 * Handles showing and hiding fields conditionally
 */
jQuery(document).ready(
	function($) {
		const commonFields = [
			'#envira-watermarking--preview--image',
			'#envira-watermarking-type-box',
			'#envira-watermarking-position-box',
			'#envira-watermarking-rotation-box',
			'#envira-watermarking-repeat-box',
			'#envira-watermarking-margin-box',
			'#envira-watermarking-existing-images-box',
			'#envira-watermarking-keep-original-box',
		].join(', ');

		const textFields = [
			'#envira-watermarking-text-box',
			'#envira-watermarking-font-box',
			'#envira-watermarking-font-size-box',
			'#envira-watermarking-color-box',
		].join(', ');

		const imageFields = [
			'#envira-watermarking-image-box',
			'#envira-watermarking-image-scale-box',
		].join(', ');

		// Show/hide elements as necessary when a conditional field is changed
		$('#envira-gallery-settings input:not([type=hidden]), #envira-gallery-settings select').conditions(
			[
				{  // Common fields
					conditions: [
						{
							element: '[name="_envira_gallery[watermarking]"]',
							type: 'checked',
							operator: 'is'
						}
					],
					actions: {
						if: {
							element: commonFields,
							action: 'show'
						},
						else: {
							element: commonFields,
							action: 'hide'
						}
					}
				},
				{  // Text fields
					conditions: [
						{
							element: '[name="_envira_gallery[watermarking_type]"]',
							type: 'value',
							operator: 'array',
							condition: ['text']
						},
						{
							element: '[name="_envira_gallery[watermarking]"]',
							type: 'checked',
							operator: 'is'
						}
					],
					actions: {
						if: {
							element: textFields,
							action: 'show'
						},
						else: {
							element: textFields,
							action: 'hide'
						}
					}
				},
				{ // Image fields
					conditions: [
						{
							element: '[name="_envira_gallery[watermarking_type]"]',
							type: 'value',
							operator: 'array',
							condition: ['image']
						},
						{
							element: '[name="_envira_gallery[watermarking]"]',
							type: 'checked',
							operator: 'is'
						}
					],
					actions: {
						if: {
							element: imageFields,
							action: 'show'
						},
						else: {
							element: imageFields,
							action: 'hide'
						}
					}
				},
				{ // Hide margin on repeat check
					conditions: [
						{
							element: '[name="_envira_gallery[watermarking_repeat]"]',
							type: 'checked',
							operator: 'is'
						},
					],
					actions: {
						if: {
							element: '#envira-watermarking-margin-box',
							action: 'hide'
						},
						else: {
							element: '#envira-watermarking-margin-box',
							action: 'show'
						}
					}
				},
				{
					conditions: [
						{
							element: '[name="_envira_gallery[watermarking]"]',
							type: 'checked',
							operator: '!is'
						}
					],
					actions: {
						if: {
							element: '#envira-watermarking-margin-box',
							action: 'hide'
						}
					}
				},
			]
		);
	}
);
