/**
 * @var EnviraWatermarking Defined in localize script.
 */

EnviraWatermarking.hidePreview = (warning) => {
	const warningParagraph = document.querySelector('#envira-watermarking--preview--warning');
	warningParagraph.innerHTML = warning;
	warningParagraph.style.display = 'block';
	document.querySelector('#envira-watermarking--preview--image-img').style.display = 'none';
	document.querySelector('.envira-watermarking-spinner').style.display = 'none';
};

EnviraWatermarking.generateImageFromText = (text, font, fontSize, color) => {
	// Create a canvas element
	const canvas = document.createElement('canvas');
	const context = canvas.getContext('2d');

	// Get canvas size.
	context.font = `${fontSize}px ${font}`;
	const textWidth = context.measureText(text).width;
	const textHeight = fontSize * 1.1;

	// Resize canvas.
	context.canvas.width = textWidth;
	context.canvas.height = textHeight;

	// Configure canvas after resize.
	context.font = `${fontSize}px ${font}`;
	context.fillStyle = color;
	context.textAlign = 'center';
	context.textBaseline = 'middle';

	// Draw the text.
	context.fillText(text, textWidth / 2, textHeight / 2);

	return canvas.toDataURL('image/png');
}

EnviraWatermarking.generatePreview = () => {
	const enabled = document.querySelector('#envira-watermarking-enabled').checked;
	if(!enabled){
		return;
	}

	const warningParagraph = document.querySelector('#envira-watermarking--preview--warning');
	const previewImg = document.querySelector('#envira-watermarking--preview--image-img');
	const spinner = document.querySelector('.envira-watermarking-spinner');

	const hidePreview = (warning) => {
		warningParagraph.innerHTML = warning;
		warningParagraph.style.display = 'block';
		previewImg.style.display = 'none';
		previewImg.style.filter = 'none';
		spinner.style.display = 'none';
	}

	const firstImageId = document.querySelector('#envira-gallery-output li')?.id || 0;
	if(!firstImageId){
		hidePreview(EnviraWatermarking.emptyImageWarning);
		return;
	}

	const type = document.querySelector('#envira-watermarking-type').value;
	const watermarkID = document.querySelector('#envira-watermarking-image-ID')?.value || 0;

	if('image' === type && parseInt(watermarkID) === 0){
		hidePreview(EnviraWatermarking.emptyWatermarkWarning);
		return;
	}

	const text = document.querySelector('#envira-watermarking-text').value;

	if('text' === type && '' === (text ?? '')){
		hidePreview(EnviraWatermarking.emptyTextWarning);
		return;
	}

	// Remove warnings
	warningParagraph.style.display = 'none';

	// Processing effect.
	if(previewImg.style.display !== 'none'){
		spinner.style.display = 'block';
	}

	previewImg.style.filter = 'brightness(50%)';

	const webSafeFonts = [
		'Arial',
		'Verdana',
		'Trebuchet MS',
		'Times New Roman',
		'Georgia',
		'Garamond',
		'Courier New',
		'Brush Script MT',
	];

	const scale = document.querySelector('#envira-watermarking-image-scale').value;
	const position = document.querySelector('#envira-watermarking-position').value;
	const rotation = document.querySelector('#envira-watermarking-rotation').value;
	const repeat = document.querySelector('#envira-watermarking-repeat').checked ? 1 : 0;
	const margin = document.querySelector('#envira-watermarking-margin').value;
	const font = document.querySelector('#envira-watermarking-font').value.replace('_', ' ');
	const fontSize = document.querySelector('#envira-watermarking-font-size').value;
	const color = document.querySelector('#envira-watermarking-color').value;
	const keepOriginal = document.querySelector('#envira-watermarking-keep-original').checked ? 1 : 0;

	const fontDescriptionElm = document.querySelector('#envira-watermarking-font-box p.description');

	const doAjax = () => {
		let watermarkAsData = '';

		if('text' === type){
			const textPreviewElement = document.querySelector('img.envira-watermarking-text-preview');
			const textImageData = document.querySelector('#envira-watermarking-text-image');

			watermarkAsData = EnviraWatermarking.generateImageFromText(text, font, fontSize, color);

			// Add watermark text preview.
			if(null === textPreviewElement){
				const img = document.createElement('img');
				img.classList.add('envira-watermarking-text-preview');
				img.src = watermarkAsData;
				textImageData.value = watermarkAsData;
				document.querySelector('#envira-watermarking-text').insertAdjacentElement('beforebegin', img);
			} else {
				textPreviewElement.src = watermarkAsData;
				textPreviewElement.style.display = 'block';
				textImageData.value = watermarkAsData;
			}
		}

		const nonce = document.getElementById('envira_watermarking_nonce').value;

		jQuery.ajax({
			url: ajaxurl,
			type: 'POST',
			data: {
				action: 'envira_watermarking_preview',
				security: nonce,
				type,
				scale,
				position,
				rotation,
				repeat,
				margin,
				keepOriginal,
				watermarkID,
				watermarkAsData,
				firstImageId,
			},
			success: function(response) {
				previewImg.style.filter = 'none';
				previewImg.style.display = 'block';
				previewImg.src = response.data.url;
				spinner.style.display = 'none';
			},
			error: function(error) {
				hidePreview(`<span style="color:darkred; font-weight: bolder">${error.responseJSON.data}</span>`);
			},
		});
	}

	if('text' === type && !webSafeFonts.includes(font)){
		WebFont.load({
			google: {
				families: [font]
			},
			active: () => {
				fontDescriptionElm.innerHTML = '';
				doAjax();
			},
			error: () => fontDescriptionElm.innerHTML = EnviraWatermarking.fontWarning,
			inactive: () => fontDescriptionElm.innerHTML = EnviraWatermarking.fontWarning,
		});
	} else {
		doAjax();
	}
};

EnviraWatermarking.colorPicker = () => {
	let timeout = null;

	jQuery('.envira-color-picker').wpColorPicker({
		change: () => {
			if(timeout){
				clearTimeout(timeout);
			}
			timeout = setTimeout(EnviraWatermarking.generatePreview, 300);
		},
	});
}

EnviraWatermarking.mediaLibrary = () => {
	document.querySelector('.insert-media-url').addEventListener('click', (event) => {
		event.preventDefault();

		const attachmentIDField = event.target.dataset.id;
		const attachmentURLField = event.target.dataset.editor;

		const mediaManager = wp.media({
			frame: 'post',
			title: 'Insert Media',
			multiple: false,
			library: {
				type: ['image/jpeg', 'image/png', 'image/webp']
			}
		});
		mediaManager.open();

		mediaManager.on('insert', selection => {
			selection.map((attachment) => {
				if(attachmentIDField){
					document.querySelector(`input#${attachmentIDField}`).value = attachment.get('id');
				}

				if(attachmentURLField){
					document.querySelector(`input#${attachmentURLField}`).value = attachment.get('url');
				}

				document.querySelector('span.envira-watermarking-image').innerHTML = `<img src="${attachment.get('url')}" />`;
				EnviraWatermarking.generatePreview();
			});
		});
	});
}

EnviraWatermarking.generateWatermark = () => {
	// Generate watermark on watermark tab load. (Maybe extract as function and add it directly to the main metabox.js file)
	new MutationObserver((mutations) => {
		mutations.forEach(mutation => {
			if(mutation.type !== 'attributes' && mutation.attributeName !== 'class') return;
			if('envira-active' === mutation.target.className){
				EnviraWatermarking.generatePreview();
			}
		});
	}).observe(document.querySelector('li.envira-watermarking a'), {attributes: true});

	// Generate watermark settings change.
	document.querySelectorAll('.envira-watermarking--fields input, .envira-watermarking--fields select').forEach((element) => {
		element.addEventListener('change', EnviraWatermarking.generatePreview);
	});
};

EnviraWatermarking.mediaLibrary();
EnviraWatermarking.colorPicker();
EnviraWatermarking.generateWatermark();
