<?php
/**
 * Metabox class.
 *
 * @since 1.0.0
 *
 * @package Envira_Watermarking
 * @author  Envira Team
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Metabox class.
 *
 * @since 1.0.0
 *
 * @package Envira_Watermarking
 * @author  Envira Team
 */
class Envira_Watermarking_Metaboxes {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var null|self
	 */
	public static $instance = null;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Holds the common class object.
	 *
	 * @var null|Envira_Watermarking_Common
	 */
	public $common = null;

	/**
	 * Holds the metaboxes class object.
	 *
	 * @var null|Envira_Gallery_Metaboxes
	 */
	public $metaboxes = null;

	/**
	 * Holds the meta key.
	 *
	 * @var string
	 */
	public $key = '_envira_gallery';

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		// Get instances.
		$this->base      = Envira_Watermarking::get_instance();
		$this->common    = Envira_Watermarking_Common::get_instance();
		$this->metaboxes = Envira_Gallery_Metaboxes::get_instance();

		add_action( 'envira_gallery_metabox_scripts', [ $this, 'metabox_scripts' ] );

		// Envira Gallery.
		add_action( 'envira_gallery_metabox_styles', [ $this, 'css' ] );
		add_filter( 'envira_gallery_tab_nav', [ $this, 'tabs' ] );
		add_action( 'envira_gallery_tab_watermarking', [ $this, 'watermarking_box' ] );
		add_filter( 'envira_gallery_save_settings', [ $this, 'gallery_settings_save' ], 10, 2 );
	}

	/**
	 * Initializes scripts for the metabox admin.
	 *
	 * @since 1.0.0
	 */
	public function metabox_scripts() {
		// Conditional Fields.
		wp_register_script(
			$this->base->plugin_slug . '-conditional-fields-script',
			envira_script( 'assets/js/min/conditional-fields-min.js', $this->base->file ),
			[ 'jquery', Envira_Gallery::get_instance()->plugin_slug . '-conditional-fields-script' ],
			$this->base->version,
			true
		);
		wp_enqueue_script( $this->base->plugin_slug . '-conditional-fields-script' );

		// Color picker.
		wp_register_script( 'wp-color-picker-alpha', envira_script( 'assets/js/min/color-picker-min.js', $this->base->file ), [ 'wp-color-picker' ], $this->base->version, true );
		wp_enqueue_script( 'wp-color-picker-alpha' );

		// Metabox script.
		wp_register_script( $this->base->plugin_slug . '-metabox', envira_script( 'assets/js/min/metabox-min.js', $this->base->file ), [ 'jquery' ], $this->base->version, true );
		wp_localize_script(
			$this->base->plugin_slug . '-metabox',
			'EnviraWatermarking',
			[
				'emptyImageWarning'     => esc_html__( 'Add images to the gallery to view watermarking preview.', 'envira-watermarking' ),
				'emptyWatermarkWarning' => esc_html__( 'Add watermark to view watermarking preview.', 'envira-watermarking' ),
				'emptyTextWarning'      => esc_html__( 'Add text to view watermarking preview.', 'envira-watermarking' ),
				'fontWarning'           => esc_html__( 'Font is not available', 'envira-watermarking' ),
			]
		);
		wp_enqueue_script( $this->base->plugin_slug . '-metabox' );

		// Enqueue web font loader.
		wp_register_script( 'envira-webfont-loader', '//ajax.googleapis.com/ajax/libs/webfont/1.6.26/webfont.js', [], $this->base->version, true );
		wp_enqueue_script( 'envira-webfont-loader' );
	}

	/**
	 * Enqueue CSS for the Metabox
	 */
	public function css() {
		wp_enqueue_style( $this->base->plugin_slug . '-metabox', plugins_url( 'assets/css/watermarking-admin.css', $this->base->file ), false, $this->base->version );
	}

	/**
	 * Adds a tab for this Addon
	 *
	 * @since 1.0.0
	 *
	 * @param array $tabs Tabs.
	 * @return array Tabs
	 */
	public function tabs( $tabs ) {
		if ( $this->metaboxes->get_config( 'type' ) === 'dynamic' ) {
			return $tabs;
		}

		$tabs['watermarking'] = __( 'Watermarking', 'envira-watermarking' );
		return $tabs;
	}

	/**
	 * Adds addon settings UI to the Watermarking tab
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	public function watermarking_box( $post ) {
		wp_nonce_field( 'envira_watermarking_save_settings', 'envira_watermarking_nonce' );
		?>
		<div id="envira-watermarking">
			<p class="envira-intro">
				<?php esc_html_e( 'Watermarking Gallery Settings', 'envira-watermarking' ); ?>
				<small>
					<?php esc_html_e( 'The settings below adjust the Watermarking options for the Gallery output.', 'envira-watermarking' ); ?>

					<?php if ( apply_filters( 'envira_whitelabel', false ) ) : ?>
						<?php do_action( 'envira_watermarking_whitelabel_tab_helptext' ); ?>
					<?php else : ?>
						<br/>
						<?php esc_html_e( 'Need some help?', 'envira-watermarking' ); ?>
						<a href="http://enviragallery.com/docs/watermarking-addon/" class="envira-doc" target="_blank" rel="noopener">
							<?php esc_html_e( 'Read the Documentation', 'envira-watermarking' ); ?>
						</a>
						or
						<a href="https://www.youtube.com/embed/Us4HAa__FEQ/?rel=0" class="envira-video" target="_blank" rel="noopener">
							<?php esc_html_e( 'Watch a Video', 'envira-watermarking' ); ?>
						</a>
					<?php endif; ?>
				</small>
			</p>
			<div class="envira-watermarking--settings">
				<div class="envira-watermarking--fields">
					<table class="form-table">
						<tbody>
						<tr id="envira-watermarking-enabled-box">
							<th scope="row">
								<label for="envira-watermarking-enabled"><?php esc_html_e( 'Enable Watermarking?', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<input id="envira-watermarking-enabled" type="checkbox" name="<?php echo esc_attr( $this->key ); ?>[watermarking]"
										value="1" <?php checked( $this->metaboxes->get_config( 'watermarking', envira_get_config_default( 'watermarking' ) ), 1 ); ?> />
								<span class="description">
								<?php
								esc_html_e(
									'Enables or disables the Watermarking of images uploaded to this Gallery.',
									'envira-watermarking'
								);
								?>
										</span>
							</td>
						</tr>
						<tr id="envira-watermarking-type-box">
							<th scope="row">
								<label for="envira-watermarking-type"><?php esc_html_e( 'Select watermark type', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<select id="envira-watermarking-type" name="<?php echo esc_attr( $this->key ); ?>[watermarking_type]">
									<?php foreach ( $this->common->get_types() as $type ) : ?>
										<option value="<?php echo esc_html( $type['value'] ); ?>"
											<?php
											selected(
												$type['value'],
												$this->metaboxes->get_config( 'watermarking_type', envira_get_config_default( 'watermarking_type' ) )
											);
											?>
										>
											<?php echo esc_html( $type['name'] ); ?>
										</option>
									<?php endforeach; ?>
								</select>
							</td>
						</tr>
						<?php $this->text_fields(); ?>
						<?php $this->image_fields(); ?>
						<tr id="envira-watermarking-position-box">
							<th scope="row">
								<label for="envira-watermarking-position"><?php esc_html_e( 'Position', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<select id="envira-watermarking-position" name="<?php echo esc_attr( $this->key ); ?>[watermarking_position]">
									<?php foreach ( (array) $this->common->get_positions() as $data ) : ?>
										<option value="<?php echo esc_html( $data['value'] ); ?>"
											<?php
											selected(
												$data['value'],
												$this->metaboxes->get_config(
													'watermarking_position',
													envira_get_config_default( 'watermarking_position' )
												)
											);
											?>
										><?php echo esc_html( $data['name'] ); ?></option>
									<?php endforeach; ?>
								</select>
								<p class="description"><?php esc_html_e( 'Define where to position the watermark over the image.', 'envira-watermarking' ); ?></p>
							</td>
						</tr>
						<tr id="envira-watermarking-rotation-box">
							<th scope="row">
								<label for="envira-watermarking-rotation"><?php esc_html_e( 'Rotation', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<input
									id="envira-watermarking-rotation"
									type="number"
									min="0"
									max="360"
									name="<?php echo esc_attr( $this->key ); ?>[watermarking_rotation]"
									value="<?php echo esc_attr( $this->metaboxes->get_config( 'watermarking_rotation', envira_get_config_default( 'watermarking_rotation' ) ) ); ?>"/> <span class="envira-unit">
											<?php
											esc_html_e(
												'degrees',
												'envira-watermarking'
											);
											?>
										</span>
								<p class="description"><?php esc_html_e( 'Sets the rotation of the watermark.', 'envira-watermarking' ); ?></p>
							</td>
						</tr>
						<tr id="envira-watermarking-repeat-box">
							<th scope="row">
								<label for="envira-watermarking-repeat"><?php esc_html_e( 'Repeat', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<input id="envira-watermarking-repeat" type="checkbox" name="<?php echo esc_attr( $this->key ); ?>[watermarking_repeat]"
										value="1" <?php checked( $this->metaboxes->get_config( 'watermarking_repeat', envira_get_config_default( 'watermarking_repeat' ) ), 1 ); ?> />
								<span class="description">
								<?php
								esc_html_e(
									'If enabled the watermark will repeat vertically and horizontally throughout the image',
									'envira-watermarking'
								);
								?>
										</span>
							</td>
						</tr>
						<tr id="envira-watermarking-margin-box">
							<th scope="row">
								<label for="envira-watermarking-margin"><?php esc_html_e( 'Margin', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<input
									id="envira-watermarking-margin"
									type="number"
									name="<?php echo esc_attr( $this->key ); ?>[watermarking_margin]"
									value="<?php echo esc_attr( $this->metaboxes->get_config( 'watermarking_margin', envira_get_config_default( 'watermarking_margin' ) ) ); ?>"/> <span class="envira-unit">
											<?php
											esc_html_e(
												'px',
												'envira-watermarking'
											);
											?>
										</span>
								<p class="description">
								<?php
								esc_html_e(
									'Sets the space between the edge of the image and the watermark.',
									'envira-watermarking'
								);
								?>
										</p>
							</td>
						</tr>
						<tr id="envira-watermarking-keep-original-box">
							<th scope="row">
								<label for="envira-watermarking-keep-original"><?php esc_html_e( 'Keep original images?', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<input id="envira-watermarking-keep-original" type="checkbox" name="<?php echo esc_attr( $this->key ); ?>[watermarking_keep_original]"
										value="1" <?php checked( $this->metaboxes->get_config( 'watermarking_keep_original', envira_get_config_default( 'watermarking_keep_original' ) ), 1 ); ?> />
								<p
									class="description">
									<?php
									esc_html_e(
										'When enabled, Envira Gallery will automatically create a copy of your original image without your watermarking settings applied.',
										'envira-watermarking'
									);
									?>
										</p>
							</td>
						</tr>
						<tr id="envira-watermarking-existing-images-box">
							<th scope="row">
								<label for="envira-watermarking-watermarking-existing-images"><?php esc_html_e( 'Apply Watermark to Existing Images', 'envira-watermarking' ); ?></label>
							</th>
							<td>
								<input
									id="envira-watermarking-existing-images"
									type="checkbox"
									name="<?php echo esc_attr( $this->key ); ?>[watermarking_existing_images]"
									value="1" <?php checked( $this->metaboxes->get_config( 'watermarking_existing_images', envira_get_config_default( 'watermarking_existing_images' ) ), 1 ); ?>
								/>
								<p
									class="description">
									<?php
									esc_html_e(
										'Select this option to watermark any existing images inside this gallery. Any images inside this gallery with a watermark will not be changed.',
										'envira-watermarking'
									);
									?>
										</p>
							</td>
						</tr>
						</tbody>
					</table>
				</div>
				<div class="envira-watermarking--preview" id="envira-watermarking-preview">
					<div id="envira-watermarking--preview--image" class="envira-watermarking--preview--image">
						<p id="envira-watermarking--preview--warning"style="display: none"><?php esc_html_e( 'Add images to the gallery to view watermarking preview', 'envira-gallery' ); ?></p>
						<img
							src=""
							alt="<?php esc_attr_e( 'Watermark preview', 'envira-gallery' ); ?>"
							class="envira-watermarking--preview--image-img"
							id="envira-watermarking--preview--image-img"
							style="display: none;"
						/>
						<div class="envira-watermarking-spinner" style="display: none"></div>
					</div>
				</div>
			</div>
		</div>
		<?php
	}

	/** Text fields */
	private function text_fields() {
		?>
		<tr id="envira-watermarking-text-box">
			<th scope="row">
				<label for="envira-watermarking-text"><?php esc_html_e( 'Text', 'envira-watermarking' ); ?></label>
			</th>
			<td>
				<input
					type="text"
					id="envira-watermarking-text"
					name="<?php echo esc_attr( $this->key ); ?>[watermarking_text]"
					class="envira-watermarking-text-fields"
					value="<?php echo esc_attr( $this->metaboxes->get_config( 'watermarking_text', envira_get_config_default( 'watermarking_text' ) ) ); ?>"
				>
				<input type="hidden" id="envira-watermarking-text-image" name="<?php echo esc_attr( $this->key ); ?>[watermarking_text_image]" />
			</td>
		</tr>
		<tr id="envira-watermarking-font-box">
			<th scope="row">
				<label for="envira-watermarking-font"><?php esc_html_e( 'Select font', 'envira-watermarking' ); ?></label>
			</th>
			<td>
				<select
					id="envira-watermarking-font"
					name="<?php echo esc_attr( $this->key ); ?>[watermarking_font]"
					class="envira-watermarking-text-fields"
				>
					<?php foreach ( $this->common->get_fonts() as $name ) : ?>
						<option value="<?php echo esc_attr( $name ); ?>"
							<?php
							selected(
								$name,
								$this->metaboxes->get_config( 'watermarking_font', envira_get_config_default( 'watermarking_font' ) )
							);
							?>
						>
						<?php echo esc_html( $name ); ?>
						</option>
					<?php endforeach; ?>
				</select>
				<p class="description" style="color:darkred;"></p>
			</td>
		</tr>
		<tr id="envira-watermarking-font-size-box">
			<th scope="row">
				<label for="envira-watermarking-font-size"><?php esc_html_e( 'Font Size', 'envira-watermarking' ); ?></label>
			</th>
			<td>
				<input
					id="envira-watermarking-font-size"
					type="number"
					name="<?php echo esc_attr( $this->key ); ?>[watermarking_font_size]"
					class="envira-watermarking-text-fields"
					value="<?php echo esc_attr( $this->metaboxes->get_config( 'watermarking_font_size', envira_get_config_default( 'watermarking_font_size' ) ) ); ?>"/> <span class="envira-unit">
							<?php esc_html_e( 'px', 'envira-watermarking' ); ?>
						</span>
				<p class="description"><?php esc_html_e( 'Sets the font size of the watermark text.', 'envira-watermarking' ); ?></p>
			</td>
		</tr>
		<tr id="envira-watermarking-color-box">
			<th scope="row">
				<label for="envira-watermarking-color"><?php esc_html_e( 'Color', 'envira-watermarking' ); ?></label>
			</th>
			<td>
				<input
					type="text"
					id="envira-watermarking-color"
					name="<?php echo esc_attr( $this->key ); ?>[watermarking_color]"
					class="envira-color-picker envira-watermarking-text-fields"
					value="<?php echo esc_html( $this->metaboxes->get_config( 'watermarking_color', envira_get_config_default( 'watermarking_color' ) ) ); ?>"
					data-alpha-enabled="true"
					data-alpha-color-type="rgba"
					data-alpha-default-color="rgba(0,175,85,0.77)"
				/>
				<p class="description"><?php esc_html_e( 'Sets the color of the watermark text.', 'envira-watermarking' ); ?></p>
			</td>
		</tr>
		<?php
	}

	/** Image fields */
	private function image_fields() {
		?>
		<tr id="envira-watermarking-image-box">
			<th scope="row">
				<label for="envira-watermarking-image"><?php esc_html_e( 'Watermark', 'envira-watermarking' ); ?></label>
			</th>
			<td>
				<!-- Stores ID and image URL -->
				<input type="hidden" id="envira-watermarking-image-ID" name="<?php echo esc_attr( $this->key ); ?>[watermarking_image_id]"
						value="<?php echo esc_html( $this->metaboxes->get_config( 'watermarking_image_id', envira_get_config_default( 'watermarking_image_id' ) ) ); ?>"
				/>

				<!-- Existing Image -->
				<span class="image envira-watermarking-image">
				<?php
				$image_id = $this->metaboxes->get_config( 'watermarking_image_id', envira_get_config_default( 'watermarking_image_id' ) );
				if ( ! empty( $image_id ) ) {
					// Get image.
					$image = wp_get_attachment_image_src( $image_id, 'thumbnail' );
					if ( is_array( $image ) ) {
						?>
						<img src="<?php echo esc_url( $image[0] ); ?>" alt="<?php esc_attr_e( 'Watermarking image', 'envira-watermarking' ); ?>"/>
						<?php
					}
				}
				?>
			</span>

				<span class="wp-media-buttons envira-watermarking-button" style="float:none;">
				<a href="#" class="button insert-media-url add_media" data-id="envira-watermarking-image-ID">
					<span class="wp-media-buttons-icon"></span>
					<?php

					if ( ! empty( $image_id ) ) {
						esc_html_e( 'Change Watermark', 'envira-watermarking' );
					} else {
						esc_html_e( 'Choose Watermark', 'envira-watermarking' );
					}
					?>
				</a>
			</span>

				<span class="description"><?php esc_html_e( 'The image to use as the watermark.', 'envira-watermarking' ); ?></span>
			</td>
		</tr>
		<tr id="envira-watermarking-image-scale-box">
			<th scope="row">
				<label for="envira-watermarking-image-scale"><?php esc_html_e( 'Image scale', 'envira-watermarking' ); ?></label>
			</th>
			<td>
				<input
					id="envira-watermarking-image-scale"
					type="number"
					min="-100"
					max="500"
					name="<?php echo esc_attr( $this->key ); ?>[watermarking_image_scale]"
					value="<?php echo esc_attr( $this->metaboxes->get_config( 'watermarking_image_scale', envira_get_config_default( 'watermarking_image_scale' ) ) ); ?>"/> <span class="envira-unit">
					<?php
					esc_html_e(
						'%',
						'envira-watermarking'
					);
					?>
				</span>
				<p class="description"><?php esc_html_e( 'Set the watermark image scale', 'envira-watermarking' ); ?></p>
			</td>
		</tr>
		<?php
	}

	/**
	 * Saves the addon's settings for Galleries.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function gallery_settings_save( $settings, $post_id ) {

		if (
			! isset( $_POST[ $this->key ], $_POST['envira_watermarking_nonce'] )
			|| ! wp_verify_nonce( sanitize_key( $_POST['envira_watermarking_nonce'] ), 'envira_watermarking_save_settings' )
		) {
			return $settings;
		}

		$post_sanitized = array_map( 'sanitize_text_field', wp_unslash( $_POST[ $this->key ] ) );

		$settings['config']['watermarking']                 = (bool) ( $post_sanitized['watermarking'] ?? false );
		$settings['config']['watermarking_image_id']        = absint( $post_sanitized['watermarking_image_id'] ?? 0 );
		$settings['config']['watermarking_position']        = (string) ( $post_sanitized['watermarking_position'] ?? envira_get_config_default( 'watermarking_position' ) );
		$settings['config']['watermarking_margin']          = (int) ( $post_sanitized['watermarking_margin'] ?? 0 );
		$settings['config']['watermarking_text']            = (string) ( $post_sanitized['watermarking_text'] ?? '' );
		$settings['config']['watermarking_font']            = (string) ( $post_sanitized['watermarking_font'] ?? envira_get_config_default( 'watermarking_font' ) );
		$settings['config']['watermarking_font_size']       = absint( $post_sanitized['watermarking_font_size'] ?? 0 );
		$settings['config']['watermarking_color']           = (string) ( $post_sanitized['watermarking_color'] ?? '' );
		$settings['config']['watermarking_rotation']        = (int) ( $post_sanitized['watermarking_rotation'] ?? 0 );
		$settings['config']['watermarking_repeat']          = (bool) ( $post_sanitized['watermarking_repeat'] ?? false );
		$settings['config']['watermarking_image_scale']     = (int) ( $post_sanitized['watermarking_image_scale'] ?? 0 );
		$settings['config']['watermarking_type']            = (string) ( $post_sanitized['watermarking_type'] ?? 'text' );
		$settings['config']['watermarking_keep_original']   = (bool) ( $post_sanitized['watermarking_keep_original'] ?? false );
		$settings['config']['watermarking_existing_images'] = (bool) ( $post_sanitized['watermarking_existing_images'] ?? false );

		$text_watermark_image = (string) ( $post_sanitized['watermarking_text_image'] ?? '' );

		update_post_meta( $post_id, '_envira_gallery_watermarking_text_image', $text_watermark_image );

		if ( ! $settings['config']['watermarking_existing_images'] ) {
			return $settings;
		}

		// Check we have some gallery images.
		if ( ! isset( $settings['gallery'] ) || ! is_array( $settings['gallery'] ) ) {
			return $settings;
		}

		// Iterate through gallery images.
		foreach ( $settings['gallery'] as $image_id => $image ) {
			$filename = basename( get_attached_file( $image_id ) );
			$new_name = Envira_Watermarking_Process::watermark_gallery_item( $image_id, $post_id, $settings );
			if ( ! $new_name ) {
				continue;
			}

			$settings['gallery'][ $image_id ]['link']  = str_replace( $filename, $new_name, $settings['gallery'][ $image_id ]['link'] );
			$settings['gallery'][ $image_id ]['src']   = str_replace( $filename, $new_name, $settings['gallery'][ $image_id ]['src'] );
			$settings['gallery'][ $image_id ]['thumb'] = str_replace( $filename, $new_name, $settings['gallery'][ $image_id ]['thumb'] );
		}

		// Return settings.
		return $settings;
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return self The Envira_Watermarking_Metaboxes object.
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}

// Load the metabox class.
$envira_watermarking_metaboxes = Envira_Watermarking_Metaboxes::get_instance();
