<?php
/**
 * Common class.
 *
 * @since 1.0.0
 *
 * @package Envira_Watermarking
 * @author  Envira Team
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Common class.
 *
 * @since 1.0.0
 *
 * @package Envira_Watermarking
 * @author  Envira Team
 */
class Envira_Watermarking_Common {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		add_filter( 'envira_gallery_defaults', [ $this, 'defaults' ], 10, 2 );
	}

	/**
	 * Adds the default settings for this addon.
	 *
	 * @since 1.0.0
	 *
	 * @param array $defaults  Array of default config values.
	 * @param int   $post_id     The current post ID.
	 * @return array $defaults Amended array of default config values.
	 */
	public function defaults( $defaults, $post_id ) {

		// Add default settings to main defaults array.
		$defaults['watermarking']                 = 0;
		$defaults['watermarking_color']           = 'rgba(0,175,85,0.77)';
		$defaults['watermarking_font']            = 'Oleo Script';
		$defaults['watermarking_font_size']       = '150';
		$defaults['watermarking_image_id']        = '';
		$defaults['watermarking_image_scale']     = '100';
		$defaults['watermarking_margin']          = 10;
		$defaults['watermarking_position']        = 'top-left';
		$defaults['watermarking_repeat']          = 0;
		$defaults['watermarking_rotation']        = 0;
		$defaults['watermarking_text']            = 'Some Text';
		$defaults['watermarking_type']            = 'image';
		$defaults['watermarking_keep_original']   = 1;
		$defaults['watermarking_existing_images'] = 1;
		// Return.
		return $defaults;
	}

	/**
	 * Helper method for retrieving positions.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array of positions.
	 */
	public function get_positions() {

		$positions = [
			[
				'value' => 'top-left',
				'name'  => __( 'Top Left', 'envira-watermarking' ),
			],
			[
				'value' => 'top-right',
				'name'  => __( 'Top Right', 'envira-watermarking' ),
			],
			[
				'value' => 'center',
				'name'  => __( 'Center', 'envira-watermarking' ),
			],
			[
				'value' => 'bottom-left',
				'name'  => __( 'Bottom Left', 'envira-watermarking' ),
			],
			[
				'value' => 'bottom-right',
				'name'  => __( 'Bottom Right', 'envira-watermarking' ),
			],
		];

		return apply_filters( 'envira_watermarking_positions', $positions );
	}

	/** Get watermark types */
	public function get_types() {
		return [
			[
				'value' => 'text',
				'name'  => __( 'Text', 'envira-watermarking' ),
			],
			[
				'value' => 'image',
				'name'  => __( 'Image', 'envira-watermarking' ),
			],
		];
	}

	/** Get watermark fonts */
	public function get_fonts() {
		return [
			'Arial',
			'Verdana',
			'Trebuchet MS',
			'Times New Roman',
			'Georgia',
			'Garamond',
			'Courier New',
			'Brush Script MT',
		] + $this->get_google_fonts();
	}

	/** Get google fonts */
	public function get_google_fonts() {
		$plugin_file = Envira_Watermarking::get_instance()->file;
		$fonts_file  = plugin_dir_path( $plugin_file ) . 'assets/json/fonts.json';

		return json_decode( file_get_contents( $fonts_file ), true );
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return self The Envira_Watermarking_Common object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Watermarking_Common ) ) {
			self::$instance = new Envira_Watermarking_Common();
		}

		return self::$instance;
	}
}

// Load the common class.
$envira_watermarking_common = Envira_Watermarking_Common::get_instance();
